@@ -181,17 +181,26 @@ static fsal_status_t wire_to_host(struct fsal_export *exp_hdl,
 				  struct gsh_buffdesc *fh_desc,
 				  int flags)
 {
-	struct ceph_handle_key *key = fh_desc->addr;
+	struct ceph_host_handle *hhdl = fh_desc->addr;
 
 	switch (in_type) {
 		/* Digested Handles */
 	case FSAL_DIGEST_NFSV3:
 	case FSAL_DIGEST_NFSV4:
-		/* wire handles */
-		if (key->chk_fscid)
-			fh_desc->len = sizeof(*key);
-		else
-			fh_desc->len = sizeof(key->chk_vi);
+		/*
+		 * Ganesha automatically mixes the export_id in with the
+		 * filehandle and strips that out before calling this
+		 * function.
+		 *
+		 * Note that we use a LE values in the filehandle. Earlier
+		 * versions treated those values as opaque, so this allows us to
+		 * maintain compatibility with legacy deployments (most of which
+		 * were on LE arch).
+		 */
+		hhdl->chk_ino = le64toh(hhdl->chk_ino);
+		hhdl->chk_snap = le64toh(hhdl->chk_snap);
+		hhdl->chk_fscid = le64toh(hhdl->chk_fscid);
+		fh_desc->len = sizeof(*hhdl);
 		break;
 	default:
 		return fsalstat(ERR_FSAL_SERVERFAULT, 0);
@@ -200,6 +209,44 @@ static fsal_status_t wire_to_host(struct fsal_export *exp_hdl,
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
+/**
+ * @brief extract "key" from a host handle
+ *
+ * This function extracts a "key" from a host handle.  That is, when
+ * given a handle that is extracted from wire_to_host() above, this
+ * method will extract the unique bits used to index the inode cache.
+ *
+ * @param[in]     exp_hdl Export handle
+ * @param[in,out] fh_desc Buffer descriptor.  The address of the
+ *                        buffer is given in @c fh_desc->buf and must
+ *                        not be changed.  @c fh_desc->len is the length
+ *                        of the data contained in the buffer, @c
+ *                        fh_desc->len must be updated to the correct
+ *                        size. In other words, the key has to be placed
+ *                        at the beginning of the buffer!
+ */
+fsal_status_t host_to_key(struct fsal_export *exp_hdl,
+			  struct gsh_buffdesc *fh_desc)
+{
+	struct ceph_handle_key *key = fh_desc->addr;
+
+	/*
+	 * Ganesha automatically mixes the export_id in with the actual wire
+	 * filehandle and strips that out before transforming it to a host
+	 * handle. This method is called on a host handle which doesn't have
+	 * the export_id.
+	 *
+	 * Most FSALs don't factor in the export_id with the handle_key,
+	 * but we want to do that for FSAL_CEPH, primarily because we
+	 * want to do accesses via different exports via different cephx
+	 * creds. Mix the export_id back in here.
+	 */
+	key->export_id = op_ctx->ctx_export->export_id;
+	fh_desc->len = sizeof(*key);
+
+	return fsalstat(ERR_FSAL_NO_ERROR, 0);
+}
+
 /**
  * @brief Create a handle object from a wire handle
  *
@@ -207,7 +254,7 @@ static fsal_status_t wire_to_host(struct fsal_export *exp_hdl,
  * looks like we shouldn't modify.
  *
  * @param[in]  export_pub Public export
- * @param[in]  desc       Handle buffer descriptor
+ * @param[in]  desc       Handle buffer descriptor (host handle)
  * @param[out] pub_handle The created handle
  *
  * @return FSAL status.
@@ -223,7 +270,7 @@ static fsal_status_t create_handle(struct fsal_export *export_pub,
 	/* FSAL status to return */
 	fsal_status_t status = { ERR_FSAL_NO_ERROR, 0 };
 	/* The FSAL specific portion of the handle received by the client */
-	struct ceph_handle_key *key = desc->addr;
+	struct ceph_host_handle *hhdl = desc->addr;
 	/* Ceph return code */
 	int rc = 0;
 	/* Stat buffer */
@@ -232,17 +279,20 @@ static fsal_status_t create_handle(struct fsal_export *export_pub,
 	struct ceph_handle *handle = NULL;
 	/* Inode pointer */
 	struct Inode *i;
+	vinodeno_t vi;
 
 	*pub_handle = NULL;
 
-	if (desc->len != sizeof(*key) &&
-	    desc->len != sizeof(key->chk_vi)) {
+	if (desc->len != sizeof(*hhdl)) {
 		status.major = ERR_FSAL_INVAL;
 		return status;
 	}
 
+	vi.ino.val = hhdl->chk_ino;
+	vi.snapid.val = hhdl->chk_snap;
+
 	/* Check our local cache first */
-	i = ceph_ll_get_inode(export->cmount, key->chk_vi);
+	i = ceph_ll_get_inode(export->cmount, vi);
 	if (!i) {
 		/*
 		 * Try the slow way, may not be in cache now.
@@ -250,10 +300,10 @@ static fsal_status_t create_handle(struct fsal_export *export_pub,
 		 * Currently, there is no interface for looking up a snapped
 		 * inode, so we just bail here in that case.
 		 */
-		if (key->chk_vi.snapid.val != CEPH_NOSNAP)
+		if (hhdl->chk_snap != CEPH_NOSNAP)
 			return ceph2fsal_error(-ESTALE);
 
-		rc = ceph_ll_lookup_inode(export->cmount, key->chk_vi.ino, &i);
+		rc = ceph_ll_lookup_inode(export->cmount, vi.ino, &i);
 		if (rc)
 			return ceph2fsal_error(rc);
 	}
@@ -349,6 +399,7 @@ void export_ops_init(struct export_ops *ops)
 	ops->prepare_unexport = ceph_prepare_unexport,
 	ops->release = release;
 	ops->lookup_path = lookup_path;
+	ops->host_to_key = host_to_key;
 	ops->wire_to_host = wire_to_host;
 	ops->create_handle = create_handle;
 	ops->get_fs_dynamic_info = get_fs_dynamic_info;
