@@ -45,8 +45,14 @@
 #include "nfs_exports.h"
 #include "sal_data.h"
 #include "statx_compat.h"
+#include "nfs_core.h"
 #include "linux/falloc.h"
 
+typedef enum {
+	CEPH_AS_CALLER,
+	CEPH_AS_ROOT,
+} ceph_access_t;
+
 /**
  * @brief Release an object
  *
@@ -62,8 +68,10 @@ static void ceph_fsal_release(struct fsal_obj_handle *obj_pub)
 	/* The private 'full' handle */
 	struct ceph_handle *obj =
 			container_of(obj_pub, struct ceph_handle, handle);
+	struct ceph_export *export =
+		container_of(op_ctx->fsal_export, struct ceph_export, export);
 
-	if (obj != obj->export->root)
+	if (obj != export->root)
 		deconstruct_handle(obj);
 }
 
@@ -798,21 +806,36 @@ static fsal_status_t ceph_fsal_unlink(struct fsal_obj_handle *dir_pub,
  * @param[in] openflags   Mode for open
  * @param[in] posix_flags POSIX open flags for open
  * @param[in] my_fd       The ceph_fd to open
+ * @param[in] access      CEPH_AS_CALLER or CEPH_AS_ROOT
  *
  * @return FSAL status.
  */
 
 static fsal_status_t ceph_open_my_fd(struct ceph_handle *myself,
 				     fsal_openflags_t openflags,
-				     int posix_flags, struct ceph_fd *my_fd)
+				     int posix_flags, struct ceph_fd *my_fd,
+				     ceph_access_t access)
 {
 	int rc;
 	struct ceph_export *export =
 		container_of(op_ctx->fsal_export, struct ceph_export, export);
+	/*
+	 * Optionally call fsal_ceph_ll_open() with root privileges.
+	 * Needed in reopen2() code path.
+	 */
+	struct user_cred root_creds = {
+		.caller_uid    = 0,
+		.caller_gid    = 0,
+		.caller_glen   = 0,
+		.caller_garray = NULL
+	};
+	struct user_cred *creds =
+		access == CEPH_AS_ROOT ? &root_creds : op_ctx->creds;
 
 	LogFullDebug(COMPONENT_FSAL,
-		     "my_fd = %p my_fd->fd = %p openflags = %x, posix_flags = %x",
-		     my_fd, my_fd->fd, openflags, posix_flags);
+		     "my_fd = %p my_fd->fd = %p openflags = %x,"
+		     " posix_flags = %x, access = %d",
+		     my_fd, my_fd->fd, openflags, posix_flags, access);
 
 	assert(my_fd->fd == NULL
 	       && my_fd->openflags == FSAL_O_CLOSED && openflags != 0);
@@ -822,7 +845,7 @@ static fsal_status_t ceph_open_my_fd(struct ceph_handle *myself,
 		     openflags, posix_flags);
 
 	rc = fsal_ceph_ll_open(export->cmount, myself->i, posix_flags,
-				&my_fd->fd, op_ctx->creds);
+				&my_fd->fd, creds);
 
 	if (rc < 0) {
 		my_fd->fd = NULL;
@@ -844,16 +867,24 @@ static fsal_status_t ceph_open_my_fd(struct ceph_handle *myself,
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
-static fsal_status_t ceph_close_my_fd(struct ceph_handle *handle,
-				      struct ceph_fd *my_fd)
+static fsal_status_t ceph_close_my_fd(struct ceph_fd *my_fd)
 {
-	int rc = 0;
 	fsal_status_t status = fsalstat(ERR_FSAL_NO_ERROR, 0);
+	struct ceph_export *export =
+		container_of(op_ctx->fsal_export, struct ceph_export, export);
 
 	if (my_fd->fd != NULL && my_fd->openflags != FSAL_O_CLOSED) {
-		rc = ceph_ll_close(handle->export->cmount, my_fd->fd);
-		if (rc < 0)
+		int rc = ceph_ll_close(export->cmount, my_fd->fd);
+
+		if (rc < 0) {
+			/*
+			 * We expect -ENOTCONN errors on shutdown. Ignore
+			 * them so we don't spam the logs.
+			 */
+			if (rc == -ENOTCONN && admin_shutdown)
+				rc = 0;
 			status = ceph2fsal_error(rc);
+		}
 		my_fd->fd = NULL;
 		my_fd->openflags = FSAL_O_CLOSED;
 	}
@@ -881,7 +912,7 @@ static fsal_status_t ceph_open_func(struct fsal_obj_handle *obj_hdl,
 
 	return ceph_open_my_fd(container_of(obj_hdl,
 		struct ceph_handle, handle), openflags,
-		posix_flags, (struct ceph_fd *)fd);
+		posix_flags, (struct ceph_fd *)fd, CEPH_AS_CALLER);
 }
 
 /**
@@ -896,8 +927,7 @@ static fsal_status_t ceph_open_func(struct fsal_obj_handle *obj_hdl,
 static fsal_status_t ceph_close_func(struct fsal_obj_handle *obj_hdl,
 				     struct fsal_fd *fd)
 {
-	return ceph_close_my_fd(container_of(obj_hdl,
-		struct ceph_handle, handle), (struct ceph_fd *)fd);
+	return ceph_close_my_fd((struct ceph_fd *)fd);
 }
 
 /**
@@ -926,7 +956,7 @@ static fsal_status_t ceph_fsal_close(struct fsal_obj_handle *obj_hdl)
 	if (handle->fd.openflags == FSAL_O_CLOSED)
 		status = fsalstat(ERR_FSAL_NOT_OPENED, 0);
 	else
-		status = ceph_close_my_fd(handle, &handle->fd);
+		status = ceph_close_my_fd(&handle->fd);
 
 	PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
@@ -1170,9 +1200,10 @@ static fsal_status_t ceph_fsal_open2(struct fsal_obj_handle *obj_hdl,
 		}
 
 		if (my_fd->openflags != FSAL_O_CLOSED) {
-			ceph_close_my_fd(myself, my_fd);
+			ceph_close_my_fd(my_fd);
 		}
-		status = ceph_open_my_fd(myself, openflags, posix_flags, my_fd);
+		status = ceph_open_my_fd(myself, openflags, posix_flags,
+					 my_fd, CEPH_AS_CALLER);
 
 		if (FSAL_IS_ERROR(status)) {
 			if (state == NULL) {
@@ -1243,7 +1274,7 @@ static fsal_status_t ceph_fsal_open2(struct fsal_obj_handle *obj_hdl,
 			return status;
 		}
 
-		(void) ceph_close_my_fd(myself, my_fd);
+		(void) ceph_close_my_fd(my_fd);
 
  undo_share:
 
@@ -1494,8 +1525,7 @@ static fsal_status_t ceph_fsal_open2(struct fsal_obj_handle *obj_hdl,
  fileerr:
 
 	/* Close the file we just opened. */
-	(void) ceph_close_my_fd(container_of(*new_obj,
-		struct ceph_handle, handle), my_fd);
+	(void)ceph_close_my_fd(my_fd);
 
 	/* Release the handle we just allocated. */
 	(*new_obj)->obj_ops->release(*new_obj);
@@ -1585,7 +1615,11 @@ static fsal_status_t ceph_fsal_reopen2(struct fsal_obj_handle *obj_hdl,
 
 	PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
-	status = ceph_open_my_fd(myself, openflags, posix_flags, my_fd);
+	/* Open the fd with root privileges on reopen. This is safe
+	 * because permission is checked in fsal_helper:fsal_reopen2()
+	 */
+	status = ceph_open_my_fd(myself, openflags, posix_flags, my_fd,
+				 CEPH_AS_ROOT);
 
 	if (!FSAL_IS_ERROR(status)) {
 		/* Close the existing file descriptor and copy the new
@@ -1594,7 +1628,7 @@ static fsal_status_t ceph_fsal_reopen2(struct fsal_obj_handle *obj_hdl,
 		 */
 		PTHREAD_RWLOCK_wrlock(&my_share_fd->fdlock);
 
-		ceph_close_my_fd(myself, my_share_fd);
+		ceph_close_my_fd(my_share_fd);
 		my_share_fd->fd = my_fd->fd;
 		my_share_fd->openflags = my_fd->openflags;
 
@@ -1672,8 +1706,6 @@ static void ceph_fsal_read2(struct fsal_obj_handle *obj_hdl, bool bypass,
 			    fsal_async_cb done_cb, struct fsal_io_arg *read_arg,
 			    void *caller_arg)
 {
-	struct ceph_handle *myself =
-			container_of(obj_hdl, struct ceph_handle, handle);
 	Fh *my_fd = NULL;
 	ssize_t nb_read;
 	fsal_status_t status;
@@ -1750,7 +1782,7 @@ static void ceph_fsal_read2(struct fsal_obj_handle *obj_hdl, bool bypass,
 		PTHREAD_RWLOCK_unlock(&ceph_fd->fdlock);
 
 	if (closefd)
-		(void) ceph_ll_close(myself->export->cmount, my_fd);
+		(void) ceph_ll_close(export->cmount, my_fd);
 
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
@@ -1781,8 +1813,6 @@ static void ceph_fsal_write2(struct fsal_obj_handle *obj_hdl, bool bypass,
 			     fsal_async_cb done_cb,
 			     struct fsal_io_arg *write_arg, void *caller_arg)
 {
-	struct ceph_handle *myself =
-			container_of(obj_hdl, struct ceph_handle, handle);
 	ssize_t nb_written;
 	fsal_status_t status;
 	int i, retval = 0;
@@ -1847,7 +1877,7 @@ static void ceph_fsal_write2(struct fsal_obj_handle *obj_hdl, bool bypass,
 		PTHREAD_RWLOCK_unlock(&ceph_fd->fdlock);
 
 	if (closefd)
-		(void) ceph_ll_close(myself->export->cmount, my_fd);
+		(void) ceph_ll_close(export->cmount, my_fd);
 
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
@@ -1932,7 +1962,7 @@ static fsal_status_t ceph_fsal_commit2(struct fsal_obj_handle *obj_hdl,
 	}
 
 	if (closefd)
-		(void) ceph_ll_close(myself->export->cmount, out_fd->fd);
+		(void) ceph_ll_close(export->cmount, out_fd->fd);
 
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
@@ -1969,8 +1999,6 @@ static fsal_status_t ceph_fsal_lock_op2(struct fsal_obj_handle *obj_hdl,
 					fsal_lock_param_t *request_lock,
 					fsal_lock_param_t *conflicting_lock)
 {
-	struct ceph_handle *myself =
-			container_of(obj_hdl, struct ceph_handle, handle);
 	struct flock lock_args;
 	fsal_status_t status = {0, 0};
 	int retval = 0;
@@ -2123,7 +2151,7 @@ static fsal_status_t ceph_fsal_lock_op2(struct fsal_obj_handle *obj_hdl,
 		PTHREAD_RWLOCK_unlock(&ceph_fd->fdlock);
 
 	if (closefd)
-		(void) ceph_ll_close(myself->export->cmount, my_fd);
+		(void) ceph_ll_close(export->cmount, my_fd);
 
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
@@ -2140,8 +2168,8 @@ static void ceph_deleg_cb(Fh *fh, void *vhdl)
 	struct ceph_handle *hdl =
 		container_of(obj_hdl, struct ceph_handle, handle);
 	struct gsh_buffdesc key = {
-		.addr = &hdl->key,
-		.len = sizeof(hdl->key)
+		.addr = &hdl->key.hhdl,
+		.len = sizeof(hdl->key.hhdl)
 	};
 
 	LogDebug(COMPONENT_FSAL, "Recalling delegations on %p", hdl);
@@ -2158,8 +2186,6 @@ static fsal_status_t ceph_fsal_lease_op2(struct fsal_obj_handle *obj_hdl,
 					 state_t *state, void *owner,
 					 fsal_deleg_t deleg)
 {
-	struct ceph_handle *myself =
-		container_of(obj_hdl, struct ceph_handle, handle);
 	fsal_status_t status = {0, 0};
 	int retval = 0;
 	Fh *my_fd = NULL;
@@ -2169,6 +2195,8 @@ static fsal_status_t ceph_fsal_lease_op2(struct fsal_obj_handle *obj_hdl,
 	bool bypass = false;
 	fsal_openflags_t openflags = FSAL_O_READ;
 	struct ceph_fd *ceph_fd = NULL;
+	struct ceph_export *export =
+		container_of(op_ctx->fsal_export, struct ceph_export, export);
 
 	switch (deleg) {
 	case FSAL_DELEG_NONE:
@@ -2208,14 +2236,14 @@ static fsal_status_t ceph_fsal_lease_op2(struct fsal_obj_handle *obj_hdl,
 		return status;
 	}
 
-	retval = ceph_ll_delegation(myself->export->cmount, my_fd, cmd,
-				    ceph_deleg_cb, obj_hdl);
+	retval = ceph_ll_delegation(export->cmount, my_fd, cmd, ceph_deleg_cb,
+				    obj_hdl);
 
 	if (ceph_fd)
 		PTHREAD_RWLOCK_unlock(&ceph_fd->fdlock);
 
 	if (closefd)
-		(void) ceph_ll_close(myself->export->cmount, my_fd);
+		(void) ceph_ll_close(export->cmount, my_fd);
 
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
@@ -2452,7 +2480,7 @@ static fsal_status_t ceph_fsal_close2(struct fsal_obj_handle *obj_hdl,
 	 * is operating on the fd while we close it.
 	 */
 	PTHREAD_RWLOCK_wrlock(&my_fd->fdlock);
-	status = ceph_close_my_fd(myself, my_fd);
+	status = ceph_close_my_fd(my_fd);
 	PTHREAD_RWLOCK_unlock(&my_fd->fdlock);
 
 	return status;
@@ -2485,17 +2513,19 @@ ceph_fsal_handle_to_wire(const struct fsal_obj_handle *handle_pub,
 		/* Digested Handles */
 	case FSAL_DIGEST_NFSV3:
 	case FSAL_DIGEST_NFSV4:
-		if (fh_desc->len < sizeof(handle->key)) {
+		if (fh_desc->len < sizeof(handle->key.hhdl)) {
 			LogMajor(COMPONENT_FSAL,
 				 "digest_handle: space too small for handle.  Need %zu, have %zu",
-				 sizeof(handle->key), fh_desc->len);
+				 sizeof(handle->key.hhdl), fh_desc->len);
 			return fsalstat(ERR_FSAL_TOOSMALL, 0);
 		} else {
-			if (handle->key.chk_fscid)
-				fh_desc->len = sizeof(handle->key);
-			else
-				fh_desc->len = sizeof(handle->key.chk_vi);
-			memcpy(fh_desc->addr, &handle->key, fh_desc->len);
+			struct ceph_host_handle *hhdl = fh_desc->addr;
+
+			/* See comments in wire_to_host */
+			hhdl->chk_ino = htole64(handle->key.hhdl.chk_ino);
+			hhdl->chk_snap = htole64(handle->key.hhdl.chk_snap);
+			hhdl->chk_fscid = htole64(handle->key.hhdl.chk_fscid);
+			fh_desc->len = sizeof(*hhdl);
 		}
 		break;
 
@@ -2532,8 +2562,6 @@ static fsal_status_t ceph_fsal_fallocate(struct fsal_obj_handle *obj_hdl,
 					 state_t *state, uint64_t offset,
 					 uint64_t length, bool allocate)
 {
-	struct ceph_handle *myself =
-			container_of(obj_hdl, struct ceph_handle, handle);
 	fsal_status_t status;
 	int retval = 0;
 	Fh *my_fd = NULL;
@@ -2580,7 +2608,7 @@ static fsal_status_t ceph_fsal_fallocate(struct fsal_obj_handle *obj_hdl,
 		PTHREAD_RWLOCK_unlock(&ceph_fd->fdlock);
 
 	if (closefd)
-		(void) ceph_ll_close(myself->export->cmount, my_fd);
+		(void) ceph_ll_close(export->cmount, my_fd);
 
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
