@@ -77,6 +77,9 @@ struct ceph_export {
 	struct ceph_handle *root;	/*< The root handle */
 	char *user_id;			/* cephx user_id for this mount */
 	char *secret_key;
+	char *sec_label_xattr;		/* name of xattr for security label */
+	char *fs_name;			/* filesystem name */
+	int64_t fscid;			/* Cluster fsid for named fs' */
 };
 
 struct ceph_fd {
@@ -97,14 +100,25 @@ struct ceph_state_fd {
  * The 'private' Ceph FSAL handle
  */
 
+struct ceph_host_handle {
+	uint64_t	chk_ino;
+	uint64_t	chk_snap;
+	int64_t		chk_fscid;
+} __attribute__ ((__packed__));
+
+struct ceph_handle_key {
+	/* NOTE: The ceph_host_handle MUST be first in this structure */
+	struct ceph_host_handle hhdl;
+	uint16_t export_id;
+};
+
 struct ceph_handle {
 	struct fsal_obj_handle handle;	/*< The public handle */
 	struct ceph_fd fd;
 	struct Inode *i;	/*< The Ceph inode */
 	const struct fsal_up_vector *up_ops;	/*< Upcall operations */
-	/*< The first export this handle belongs to */
-	struct ceph_export *export;
-	vinodeno_t vi;		/*< The object identifier */
+	struct ceph_handle_key key;	/*< The handle-key that includes the
+					    ceph_host_handle. */
 	struct fsal_share share;
 #ifdef CEPH_PNFS
 	uint64_t rd_issued;
@@ -141,12 +155,12 @@ struct ds {
 
 #endif				/* CEPH_PNFS */
 
-#define CEPH_SUPPORTED_ATTRS ((const attrmask_t) (ATTRS_POSIX))
+#define CEPH_SUPPORTED_ATTRS ((const attrmask_t) (ATTRS_POSIX|ATTR4_SEC_LABEL))
 
 #define CEPH_SETTABLE_ATTRIBUTES ((const attrmask_t) (			\
 	ATTR_MODE  | ATTR_OWNER | ATTR_GROUP | ATTR_ATIME	 |	\
 	ATTR_CTIME | ATTR_MTIME | ATTR_SIZE  | ATTR_MTIME_SERVER |	\
-	ATTR_ATIME_SERVER))
+	ATTR_ATIME_SERVER | ATTR4_SEC_LABEL))
 
 /* Prototypes */
 
