@@ -158,6 +158,8 @@ get_quota(struct fsal_export *exp_hdl, const char *filepath, int quota_type,
 	args.cmd = GPFS_QCMD(Q_GETQUOTA, quota_type);
 	args.qid = quota_id;
 	args.bufferP = &gpfs_quota;
+	if (op_ctx && op_ctx->client)
+		args.cli_ip = op_ctx->client->hostaddr_str;
 
 	fsal_set_credentials(op_ctx->creds);
 	if (gpfs_ganesha(OPENHANDLE_QUOTA, &args) < 0)
@@ -223,6 +225,8 @@ set_quota(struct fsal_export *exp_hdl, const char *filepath, int quota_type,
 	args.cmd = GPFS_QCMD(Q_SETQUOTA, quota_type);
 	args.qid = quota_id;
 	args.bufferP = &gpfs_quota;
+	if (op_ctx && op_ctx->client)
+		args.cli_ip = op_ctx->client->hostaddr_str;
 
 	fsal_set_credentials(op_ctx->creds);
 	if (gpfs_ganesha(OPENHANDLE_QUOTA, &args) < 0)
@@ -537,6 +541,8 @@ int gpfs_claim_filesystem(struct fsal_filesystem *fs, struct fsal_export *exp)
 		goto errout;
 	}
 
+	gpfs_fs->stop_thread = false;
+
 	if (pthread_attr_init(&attr_thr) != 0)
 		LogCrit(COMPONENT_THREAD, "can't init pthread's attributes");
 
@@ -624,6 +630,12 @@ void gpfs_unclaim_filesystem(struct fsal_filesystem *fs)
 	else
 		LogFullDebug(COMPONENT_FSAL, "Thread STOP successful");
 
+	/* Prior to calling pthread_join(), we should set stop_thread=true.
+	 * Its not being set atomically because the synchronization requirement
+	 * is not critical enough.
+	 */
+	gpfs_fs->stop_thread = true;
+
 	pthread_join(gpfs_fs->up_thread, NULL);
 	free_gpfs_filesystem(gpfs_fs);
 	fs->private_data = NULL;
