@@ -1,7 +1,7 @@
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
- * Copyright 2017-2018 Red Hat, Inc.
+ * Copyright 2017-2019 Red Hat, Inc.
  * Author: Daniel Gryniewicz  dang@redhat.com
  *
  *
@@ -190,6 +190,50 @@ void mem_export_ops_init(struct export_ops *ops)
 	ops->alloc_state = mem_alloc_state;
 }
 
+const char *str_async_type(uint32_t async_type)
+{
+	switch (async_type) {
+	case MEM_INLINE:
+		return "INLINE";
+	case MEM_RANDOM_OR_INLINE:
+		return "RANDOM_OR_INLINE";
+	case MEM_RANDOM:
+		return "RANDOM";
+	case MEM_FIXED:
+		return "FIXED";
+	}
+
+	return "UNKNOWN";
+}
+
+static struct config_item_list async_types_conf[] = {
+	CONFIG_LIST_TOK("inline",		MEM_INLINE),
+	CONFIG_LIST_TOK("fixed",		MEM_FIXED),
+	CONFIG_LIST_TOK("random",		MEM_RANDOM),
+	CONFIG_LIST_TOK("random_or_inline",	MEM_RANDOM_OR_INLINE),
+	CONFIG_LIST_EOL
+};
+
+static struct config_item mem_export_params[] = {
+	CONF_ITEM_NOOP("name"),
+	CONF_ITEM_UI32("Async_Delay", 0, 1000, 0,
+		       mem_fsal_export, async_delay),
+	CONF_ITEM_TOKEN("Async_Type", MEM_INLINE, async_types_conf,
+			mem_fsal_export, async_type),
+	CONF_ITEM_UI32("Async_Stall_Delay", 0, 1000, 0,
+		       mem_fsal_export, async_stall_delay),
+	CONFIG_EOL
+};
+
+static struct config_block mem_export_param_block = {
+	.dbus_interface_name = "org.ganesha.nfsd.config.fsal.mem-export%d",
+	.blk_desc.name = "FSAL",
+	.blk_desc.type = CONFIG_BLOCK,
+	.blk_desc.u.blk.init = noop_conf_init,
+	.blk_desc.u.blk.params = mem_export_params,
+	.blk_desc.u.blk.commit = noop_conf_commit
+};
+
 /* create_export
  * Create an export point and return a handle to it to be kept
  * in the export list.
@@ -205,6 +249,7 @@ fsal_status_t mem_create_export(struct fsal_module *fsal_hdl,
 	struct mem_fsal_export *myself;
 	int retval = 0;
 	pthread_rwlockattr_t attrs;
+	fsal_status_t fsal_status = {0, 0};
 
 	myself = gsh_calloc(1, sizeof(struct mem_fsal_export));
 
@@ -219,16 +264,25 @@ fsal_status_t mem_create_export(struct fsal_module *fsal_hdl,
 	fsal_export_init(&myself->export);
 	mem_export_ops_init(&myself->export.exp_ops);
 
+	retval = load_config_from_node(parse_node,
+				       &mem_export_param_block,
+				       myself,
+				       true,
+				       err_type);
+
+	if (retval != 0) {
+		fsal_status = posix2fsal_status(EINVAL);
+		goto err_free;	/* seriously bad */
+	}
+
 	retval = fsal_attach_export(fsal_hdl, &myself->export.exports);
 
 	if (retval != 0) {
 		/* seriously bad */
 		LogMajor(COMPONENT_FSAL,
 			 "Could not attach export");
-		free_export_ops(&myself->export);
-		gsh_free(myself);	/* elvis has left the building */
-
-		return fsalstat(posix2fsal_error(retval), retval);
+		fsal_status = posix2fsal_status(retval);
+		goto err_free;	/* seriously bad */
 	}
 
 	myself->export.fsal = fsal_hdl;
@@ -246,4 +300,81 @@ fsal_status_t mem_create_export(struct fsal_module *fsal_hdl,
 		 myself, myself->export_path);
 
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
+
+err_free:
+	free_export_ops(&myself->export);
+	gsh_free(myself);	/* elvis has left the building */
+	return fsal_status;
+}
+
+/**
+ * @brief Update an existing export
+ *
+ * This will result in a temporary fsal_export being created, and built into
+ * a stacked export.
+ *
+ * On entry, op_ctx has the original gsh_export and no fsal_export.
+ *
+ * The caller passes the original fsal_export, as well as the new super_export's
+ * FSAL when there is a stacked export. This will allow the underlying export to
+ * validate that the stacking has not changed.
+ *
+ * This function does not actually create a new fsal_export, the only purpose is
+ * to validate and update the config.
+ *
+ * @param[in]     fsal_hdl         FSAL module
+ * @param[in]     parse_node       opaque pointer to parse tree node for
+ *                                 export options to be passed to
+ *                                 load_config_from_node
+ * @param[out]    err_type         config proocessing error reporting
+ * @param[in]     original         The original export that is being updated
+ * @param[in]     updated_super    The updated super_export's FSAL
+ *
+ * @return FSAL status.
+ */
+
+fsal_status_t mem_update_export(struct fsal_module *fsal_hdl,
+				void *parse_node,
+				struct config_error_type *err_type,
+				struct fsal_export *original,
+				struct fsal_module *updated_super)
+{
+	struct mem_fsal_export myself;
+	int retval = 0;
+	struct mem_fsal_export *orig =
+		container_of(original, struct mem_fsal_export, export);
+	fsal_status_t status;
+
+	/* Check for changes in stacking by calling default update_export. */
+	status = update_export(fsal_hdl, parse_node, err_type,
+			       original, updated_super);
+
+	if (FSAL_IS_ERROR(status))
+		return status;
+
+	memset(&myself, 0, sizeof(myself));
+
+	retval = load_config_from_node(parse_node,
+				       &mem_export_param_block,
+				       &myself,
+				       true,
+				       err_type);
+
+	if (retval != 0) {
+		return posix2fsal_status(EINVAL);
+	}
+
+	/* Update the async parameters */
+	atomic_store_uint32_t(&orig->async_delay, myself.async_delay);
+	atomic_store_uint32_t(&orig->async_stall_delay,
+			      myself.async_stall_delay);
+	atomic_store_uint32_t(&orig->async_type, myself.async_type);
+
+	LogEvent(COMPONENT_FSAL,
+		 "Updated FSAL_MEM aync parameters type=%s, delay=%"PRIu32
+		 ", stall_delay=%"PRIu32,
+		 str_async_type(myself.async_type),
+		 myself.async_delay, myself.async_stall_delay);
+
+	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
