@@ -46,7 +46,7 @@
 
 
 #if ((LIBRGW_FILE_VER_MAJOR != 1) || (LIBRGW_FILE_VER_MINOR < 1) || \
-	(LIBRGW_FILE_VER_EXTRA < 2))
+	(LIBRGW_FILE_VER_EXTRA < 7))
 #error rados/rgw_file.h version unsupported (require >= 1.1.1)
 #endif
 
@@ -56,7 +56,7 @@
 
 struct rgw_fsal_module {
 	struct fsal_module fsal;
-	fsal_staticfsinfo_t fs_info;
+	struct fsal_obj_ops handle_ops;
 	char *conf_path;
 	char *name;
 	char *cluster;
@@ -104,7 +104,7 @@ struct rgw_handle {
  */
 struct rgw_open_state {
 	struct state_t gsh_open;
-	uint32_t flags;
+	fsal_openflags_t openflags;
 };
 
 /**
@@ -130,13 +130,6 @@ struct rgw_open_state {
 
 static const size_t BIGGEST_PATTERN = 1024;
 
-static inline fsal_staticfsinfo_t *rgw_staticinfo(struct fsal_module *hdl)
-{
-	struct rgw_fsal_module *myself =
-	    container_of(hdl, struct rgw_fsal_module, fsal);
-	return &myself->fs_info;
-}
-
 /* Prototypes */
 int construct_handle(struct rgw_export *export,
 		     struct rgw_file_handle *rgw_file_handle,
