@@ -192,7 +192,7 @@ cih_fhcache_inline_lookup(const struct avltree *tree,
  *
  * @return (void)
  */
-static inline bool
+static inline void
 cih_hash_key(mdcache_key_t *key,
 	     struct fsal_module *fsal,
 	     struct gsh_buffdesc *fh_desc,
@@ -213,8 +213,6 @@ cih_hash_key(mdcache_key_t *key,
 	/* hash it */
 	key->hk =
 	    CityHash64WithSeed(fh_desc->addr, fh_desc->len, 557);
-
-	return true;
 }
 
 #define CIH_GET_NONE           0x0000
@@ -326,6 +324,16 @@ cih_get_by_key_latch(mdcache_key_t *key, cih_latch_t *latch,
 
  found:
 	entry = avltree_container_of(node, mdcache_entry_t, fh_hk.node_k);
+
+	if (atomic_fetch_int32_t(&entry->lru.refcnt) == 0) {
+		/* If refcount is 0, this entry is being freed, but has not yet
+		 * been removed from the hashtable.  Don't return it */
+		LogDebug(COMPONENT_HASHTABLE_CACHE, "entry %p being freed",
+			 entry);
+		entry = NULL;
+		if (flags & CIH_GET_UNLOCK_ON_MISS)
+			cih_hash_release(latch);
+	}
  out:
 	return entry;
 }
@@ -340,12 +348,14 @@ cih_get_by_key_latch(mdcache_key_t *key, cih_latch_t *latch,
  * Insert cache entry on partition previously locked.
  *
  * @param entry [in] Entry to be inserted
+ * @param latch [in] latch held for set
+ * @param fsak [in] FSAL used to hash the key
  * @param fh_desc [in] Hash input bytes (MUST be those used previously)
  * @param flags [in] Flags
  *
- * @return Pointer to cache entry if found, else NULL
+ * @return void
  */
-static inline int
+static inline void
 cih_set_latched(mdcache_entry_t *entry, cih_latch_t *latch,
 		struct fsal_module *fsal,
 		struct gsh_buffdesc *fh_desc,
@@ -356,9 +366,7 @@ cih_set_latched(mdcache_entry_t *entry, cih_latch_t *latch,
 	/* Omit hash if you are SURE we hashed it, and that the
 	 * hash remains valid */
 	if (unlikely(!(flags & CIH_SET_HASHED)))
-		if (!cih_hash_key(&entry->fh_hk.key, fsal,
-				  fh_desc, CIH_HASH_NONE))
-			return 1;
+		cih_hash_key(&entry->fh_hk.key, fsal, fh_desc, CIH_HASH_NONE);
 
 	(void)avltree_insert(&entry->fh_hk.node_k, &cp->t);
 	entry->fh_hk.inavl = true;
@@ -369,8 +377,6 @@ cih_set_latched(mdcache_entry_t *entry, cih_latch_t *latch,
 
 	if (likely(flags & CIH_SET_UNLOCK))
 		cih_hash_release(latch);
-
-	return 0;
 }
 
 /**
@@ -389,6 +395,7 @@ cih_remove_checked(mdcache_entry_t *entry)
 	struct avltree_node *node;
 	cih_partition_t *cp =
 	    cih_partition_of_scalar(&cih_fhcache, entry->fh_hk.key.hk);
+	bool unref = false;
 	bool freed = false;
 
 	PTHREAD_RWLOCK_wrlock(&cp->lock);
@@ -403,10 +410,17 @@ cih_remove_checked(mdcache_entry_t *entry)
 					     entry->fh_hk.key.hk)] = NULL;
 		entry->fh_hk.inavl = false;
 		/* return sentinel ref */
-		freed = mdcache_lru_unref(entry);
+		unref = true;
 	}
 	PTHREAD_RWLOCK_unlock(&cp->lock);
 
+	if (unref) {
+		/* We can't unref with the lock held, in case this is the last
+		 * ref.  That will recurse into here, and try to take the lock
+		 * again. */
+		freed = mdcache_lru_unref(entry);
+	}
+
 	return freed;
 }
 
