@@ -177,6 +177,7 @@ fsal_errors_t posix2fsal_error(int posix_errorcode)
 		/* Error code that needs a retry */
 	case EAGAIN:
 	case EBUSY:
+	case ETIMEDOUT:
 #ifdef ETIME
 	case ETIME:
 #endif
@@ -206,8 +207,8 @@ fsal_errors_t posix2fsal_error(int posix_errorcode)
 
 	default:
 		LogCrit(COMPONENT_FSAL,
-			"Mapping %d(default) to ERR_FSAL_SERVERFAULT",
-			posix_errorcode);
+			"Default case mapping %s (%d) to ERR_FSAL_SERVERFAULT",
+			strerror(posix_errorcode), posix_errorcode);
 		/* other unexpected errors */
 		return ERR_FSAL_SERVERFAULT;
 
@@ -238,35 +239,6 @@ int fsal2posix_testperm(fsal_accessflags_t testperm)
 
 }
 
-/* mode bits are a uint16_t and chmod masks off type */
-
-#define S_IALLUGO (~S_IFMT & 0xFFFF)
-
-/**
- * @brief Convert FSAL mode to POSIX mode
- *
- * @param[in] fsal_mode FSAL mode to be translated
- *
- * @return The POSIX mode associated to fsal_mode.
- */
-mode_t fsal2unix_mode(uint32_t fsal_mode)
-{
-	return fsal_mode & S_IALLUGO;
-}
-
-/**
- * @brief Convert POSIX mode to FSAL mode
- *
- * @param[in] unix_mode POSIX mode to be translated
- *
- * @return FSAL mode associated with @c unix_mode
- */
-
-uint32_t unix2fsal_mode(mode_t unix_mode)
-{
-	return unix_mode & S_IALLUGO;
-}
-
 /**
  * @brief Convert POSIX object type to an FSAL object type
  *
@@ -481,11 +453,11 @@ void posix2fsal_attributes(const struct stat *buffstat,
 #endif
 	}
 
-	if (FSAL_TEST_MASK(fsalattr->valid_mask, ATTR_CHGTIME)) {
-		fsalattr->chgtime =
-			gsh_time_cmp(&fsalattr->mtime, &fsalattr->ctime) > 0 ?
-			fsalattr->mtime : fsalattr->ctime;
-		fsalattr->change = timespec_to_nsecs(&fsalattr->chgtime);
+	if (FSAL_TEST_MASK(fsalattr->valid_mask, ATTR_CHANGE)) {
+		fsalattr->change =
+			gsh_time_cmp(&fsalattr->mtime, &fsalattr->ctime) > 0
+				? timespec_to_nsecs(&fsalattr->mtime)
+				: timespec_to_nsecs(&fsalattr->ctime);
 	}
 
 	if (FSAL_TEST_MASK(fsalattr->valid_mask, ATTR_SPACEUSED))
