@@ -61,6 +61,10 @@
 #include "nfs_dupreq.h"
 #include "nfs_file_handle.h"
 
+#ifdef USE_LTTNG
+#include "gsh_lttng/nfs_rpc.h"
+#endif
+
 #define NFS_pcp nfs_param.core_param
 #define NFS_options NFS_pcp.core_options
 #define NFS_program NFS_pcp.program
@@ -90,7 +94,8 @@ static struct rpc_evchan rpc_evchan[EVCHAN_SIZE];
 
 static enum xprt_stat nfs_rpc_tcp_user_data(SVCXPRT *);
 static enum xprt_stat nfs_rpc_free_user_data(SVCXPRT *);
-static enum xprt_stat nfs_rpc_decode_request(SVCXPRT *, XDR *);
+static struct svc_req *alloc_nfs_request(SVCXPRT *xprt, XDR *xdrs);
+static void free_nfs_request(struct svc_req *req, enum xprt_stat stat);
 
 const char *xprt_stat_s[XPRT_DESTROYED + 1] = {
 	"XPRT_IDLE",
@@ -241,8 +246,12 @@ static void close_rpc_fd(void)
 	for (p = P_NFS; p < P_COUNT; p++) {
 		if (udp_socket[p] != -1)
 			close(udp_socket[p]);
+		if (udp_xprt[p])
+			SVC_DESTROY(udp_xprt[p]);
 		if (tcp_socket[p] != -1)
 			close(tcp_socket[p]);
+		if (tcp_xprt[p])
+			SVC_DESTROY(tcp_xprt[p]);
 	}
 	/* no need for special tcp_xprt[P_NFS_VSOCK] treatment */
 }
@@ -263,6 +272,9 @@ static enum xprt_stat nfs_rpc_dispatch_udp_NFS(SVCXPRT *xprt)
 	LogFullDebug(COMPONENT_DISPATCH,
 		     "NFS UDP request for SVCXPRT %p fd %d",
 		     xprt, xprt->xp_fd);
+#ifdef USE_LTTNG
+	tracepoint(nfs_rpc, before_recv, __func__, __LINE__, xprt);
+#endif
 	xprt->xp_dispatch.process_cb = nfs_rpc_valid_NFS;
 	return SVC_RECV(xprt);
 }
@@ -272,6 +284,9 @@ static enum xprt_stat nfs_rpc_dispatch_udp_MNT(SVCXPRT *xprt)
 	LogFullDebug(COMPONENT_DISPATCH,
 		     "MOUNT UDP request for SVCXPRT %p fd %d",
 		     xprt, xprt->xp_fd);
+#ifdef USE_LTTNG
+	tracepoint(nfs_rpc, before_recv, __func__, __LINE__, xprt);
+#endif
 	xprt->xp_dispatch.process_cb = nfs_rpc_valid_MNT;
 	return SVC_RECV(xprt);
 }
@@ -281,6 +296,9 @@ static enum xprt_stat nfs_rpc_dispatch_udp_NLM(SVCXPRT *xprt)
 	LogFullDebug(COMPONENT_DISPATCH,
 		     "NLM UDP request for SVCXPRT %p fd %d",
 		     xprt, xprt->xp_fd);
+#ifdef USE_LTTNG
+	tracepoint(nfs_rpc, before_recv, __func__, __LINE__, xprt);
+#endif
 	xprt->xp_dispatch.process_cb = nfs_rpc_valid_NLM;
 	return SVC_RECV(xprt);
 }
@@ -290,6 +308,9 @@ static enum xprt_stat nfs_rpc_dispatch_udp_RQUOTA(SVCXPRT *xprt)
 	LogFullDebug(COMPONENT_DISPATCH,
 		     "RQUOTA UDP request for SVCXPRT %p fd %d",
 		     xprt, xprt->xp_fd);
+#ifdef USE_LTTNG
+	tracepoint(nfs_rpc, before_recv, __func__, __LINE__, xprt);
+#endif
 	xprt->xp_dispatch.process_cb = nfs_rpc_valid_RQUOTA;
 	return SVC_RECV(xprt);
 }
@@ -359,22 +380,25 @@ const svc_xprt_fun_t tcp_dispatch[] = {
 
 void Create_udp(protos prot)
 {
-	udp_xprt[prot] =
-	    svc_dg_create(udp_socket[prot],
-			  nfs_param.core_param.rpc.max_send_buffer_size,
-			  nfs_param.core_param.rpc.max_recv_buffer_size);
-	if (udp_xprt[prot] == NULL)
-		LogFatal(COMPONENT_DISPATCH, "Cannot allocate %s/UDP SVCXPRT",
-			 tags[prot]);
+	if (udp_socket[prot] != -1) {
+		udp_xprt[prot] =
+		    svc_dg_create(udp_socket[prot],
+			nfs_param.core_param.rpc.max_send_buffer_size,
+			nfs_param.core_param.rpc.max_recv_buffer_size);
+		if (udp_xprt[prot] == NULL)
+			LogFatal(COMPONENT_DISPATCH,
+				 "Cannot allocate %s/UDP SVCXPRT", tags[prot]);
 
-	udp_xprt[prot]->xp_dispatch.rendezvous_cb = udp_dispatch[prot];
+		udp_xprt[prot]->xp_dispatch.rendezvous_cb = udp_dispatch[prot];
 
-	/* Hook xp_free_user_data (finalize/free private data) */
-	(void)SVC_CONTROL(udp_xprt[prot], SVCSET_XP_FREE_USER_DATA,
-			  nfs_rpc_free_user_data);
+		/* Hook xp_free_user_data (finalize/free private data) */
+		(void)SVC_CONTROL(udp_xprt[prot], SVCSET_XP_FREE_USER_DATA,
+				  nfs_rpc_free_user_data);
 
-	(void)svc_rqst_evchan_reg(rpc_evchan[UDP_UREG_CHAN].chan_id,
-				  udp_xprt[prot], SVC_RQST_FLAG_XPRT_UREG);
+		(void)svc_rqst_evchan_reg(rpc_evchan[UDP_UREG_CHAN].chan_id,
+					  udp_xprt[prot],
+					  SVC_RQST_FLAG_XPRT_UREG);
+	}
 }
 
 void Create_tcp(protos prot)
@@ -480,40 +504,47 @@ static int Bind_sockets_V6(void)
 
 			proto_data *pdatap = &pdata[p];
 
-			memset(&pdatap->sinaddr_udp6, 0,
-			       sizeof(pdatap->sinaddr_udp6));
-			pdatap->sinaddr_udp6.sin6_family = AF_INET6;
-			/* all interfaces */
-			pdatap->sinaddr_udp6.sin6_addr =
-			    ((struct sockaddr_in6 *)
-			    &nfs_param.core_param.bind_addr)->sin6_addr;
-			pdatap->sinaddr_udp6.sin6_port =
-			    htons(nfs_param.core_param.port[p]);
+			if (udp_socket[p] != -1) {
+				memset(&pdatap->sinaddr_udp6, 0,
+				       sizeof(pdatap->sinaddr_udp6));
+				pdatap->sinaddr_udp6.sin6_family = AF_INET6;
+				/* all interfaces */
+				pdatap->sinaddr_udp6.sin6_addr =
+				    ((struct sockaddr_in6 *)
+				    &nfs_param.core_param.bind_addr)->sin6_addr;
+				pdatap->sinaddr_udp6.sin6_port =
+				    htons(nfs_param.core_param.port[p]);
 
-			pdatap->netbuf_udp6.maxlen =
-			    sizeof(pdatap->sinaddr_udp6);
-			pdatap->netbuf_udp6.len = sizeof(pdatap->sinaddr_udp6);
-			pdatap->netbuf_udp6.buf = &pdatap->sinaddr_udp6;
+				pdatap->netbuf_udp6.maxlen =
+				    sizeof(pdatap->sinaddr_udp6);
+				pdatap->netbuf_udp6.len =
+				    sizeof(pdatap->sinaddr_udp6);
+				pdatap->netbuf_udp6.buf = &pdatap->sinaddr_udp6;
 
-			pdatap->bindaddr_udp6.qlen = SOMAXCONN;
-			pdatap->bindaddr_udp6.addr = pdatap->netbuf_udp6;
+				pdatap->bindaddr_udp6.qlen = SOMAXCONN;
+				pdatap->bindaddr_udp6.addr =
+				    pdatap->netbuf_udp6;
 
-			if (!__rpc_fd2sockinfo(udp_socket[p],
-			    &pdatap->si_udp6)) {
-				LogWarn(COMPONENT_DISPATCH,
-					 "Cannot get %s socket info for udp6 socket errno=%d (%s)",
-					 tags[p], errno, strerror(errno));
-				return -1;
-			}
+				if (!__rpc_fd2sockinfo(udp_socket[p],
+				    &pdatap->si_udp6)) {
+					LogWarn(COMPONENT_DISPATCH,
+						"Cannot get %s socket info for udp6 socket errno=%d (%s)",
+						tags[p], errno,
+						strerror(errno));
+					return -1;
+				}
 
-			rc = bind(udp_socket[p],
-			      (struct sockaddr *)pdatap->bindaddr_udp6.addr.buf,
-				  (socklen_t) pdatap->si_udp6.si_alen);
-			if (rc == -1) {
-				LogWarn(COMPONENT_DISPATCH,
-					 "Cannot bind %s udp6 socket, error %d (%s)",
-					 tags[p], errno, strerror(errno));
-				goto exit;
+				rc = bind(udp_socket[p],
+					  (struct sockaddr *)
+					  pdatap->bindaddr_udp6.addr.buf,
+					  (socklen_t) pdatap->si_udp6.si_alen);
+				if (rc == -1) {
+					LogWarn(COMPONENT_DISPATCH,
+						"Cannot bind %s udp6 socket, error %d (%s)",
+						tags[p], errno,
+						strerror(errno));
+					goto exit;
+				}
 			}
 
 			memset(&pdatap->sinaddr_tcp6, 0,
@@ -572,41 +603,48 @@ static int Bind_sockets_V4(void)
 
 			proto_data *pdatap = &pdata[p];
 
-			memset(&pdatap->sinaddr_udp, 0,
-			       sizeof(pdatap->sinaddr_udp));
-			pdatap->sinaddr_udp.sin_family = AF_INET;
-			/* all interfaces */
-			pdatap->sinaddr_udp.sin_addr.s_addr =
-			    ((struct sockaddr_in *)
-			    &nfs_param.core_param.bind_addr)->sin_addr.s_addr;
-			pdatap->sinaddr_udp.sin_port =
-			    htons(nfs_param.core_param.port[p]);
+			if (udp_socket[p] != -1) {
+				memset(&pdatap->sinaddr_udp, 0,
+				       sizeof(pdatap->sinaddr_udp));
+				pdatap->sinaddr_udp.sin_family = AF_INET;
+				/* all interfaces */
+				pdatap->sinaddr_udp.sin_addr.s_addr =
+				    ((struct sockaddr_in *)
+				    &nfs_param.core_param.bind_addr)->
+				    sin_addr.s_addr;
+				pdatap->sinaddr_udp.sin_port =
+				    htons(nfs_param.core_param.port[p]);
 
-			pdatap->netbuf_udp6.maxlen =
-			    sizeof(pdatap->sinaddr_udp);
-			pdatap->netbuf_udp6.len = sizeof(pdatap->sinaddr_udp);
-			pdatap->netbuf_udp6.buf = &pdatap->sinaddr_udp;
+				pdatap->netbuf_udp6.maxlen =
+				    sizeof(pdatap->sinaddr_udp);
+				pdatap->netbuf_udp6.len =
+				    sizeof(pdatap->sinaddr_udp);
+				pdatap->netbuf_udp6.buf = &pdatap->sinaddr_udp;
 
-			pdatap->bindaddr_udp6.qlen = SOMAXCONN;
-			pdatap->bindaddr_udp6.addr = pdatap->netbuf_udp6;
+				pdatap->bindaddr_udp6.qlen = SOMAXCONN;
+				pdatap->bindaddr_udp6.addr =
+				    pdatap->netbuf_udp6;
 
-			if (!__rpc_fd2sockinfo(udp_socket[p],
-			    &pdatap->si_udp6)) {
-				LogWarn(COMPONENT_DISPATCH,
-					"Cannot get %s socket info for udp6 socket errno=%d (%s)",
-					tags[p], errno, strerror(errno));
-				return -1;
-			}
+				if (!__rpc_fd2sockinfo(udp_socket[p],
+				    &pdatap->si_udp6)) {
+					LogWarn(COMPONENT_DISPATCH,
+						"Cannot get %s socket info for udp6 socket errno=%d (%s)",
+						tags[p], errno,
+						strerror(errno));
+					return -1;
+				}
 
-			rc = bind(udp_socket[p],
-				  (struct sockaddr *)
-				  pdatap->bindaddr_udp6.addr.buf,
-				  (socklen_t) pdatap->si_udp6.si_alen);
-			if (rc == -1) {
-				LogWarn(COMPONENT_DISPATCH,
-					"Cannot bind %s udp6 socket, error %d (%s)",
-					tags[p], errno, strerror(errno));
-				return -1;
+				rc = bind(udp_socket[p],
+					  (struct sockaddr *)
+					  pdatap->bindaddr_udp6.addr.buf,
+					  (socklen_t) pdatap->si_udp6.si_alen);
+				if (rc == -1) {
+					LogWarn(COMPONENT_DISPATCH,
+						"Cannot bind %s udp6 socket, error %d (%s)",
+						tags[p], errno,
+						strerror(errno));
+					return -1;
+				}
 			}
 
 			memset(&pdatap->sinaddr_tcp, 0,
@@ -717,14 +755,16 @@ static int alloc_socket_setopts(int p)
 
 	/* Use SO_REUSEADDR in order to avoid wait
 	 * the 2MSL timeout */
-	if (setsockopt(udp_socket[p],
-		       SOL_SOCKET, SO_REUSEADDR,
-		       &one, sizeof(one))) {
-		LogWarn(COMPONENT_DISPATCH,
-			"Bad udp socket options for %s, error %d(%s)",
-			tags[p], errno, strerror(errno));
+	if (udp_socket[p] != -1) {
+		if (setsockopt(udp_socket[p],
+			       SOL_SOCKET, SO_REUSEADDR,
+			       &one, sizeof(one))) {
+			LogWarn(COMPONENT_DISPATCH,
+				"Bad udp socket options for %s, error %d(%s)",
+				tags[p], errno, strerror(errno));
 
-		return -1;
+			return -1;
+		}
 	}
 
 	if (setsockopt(tcp_socket[p],
@@ -785,14 +825,16 @@ static int alloc_socket_setopts(int p)
 		}
 	}
 
-	/* We prefer using non-blocking socket
-	 * in the specific case */
-	if (fcntl(udp_socket[p], F_SETFL, FNDELAY) == -1) {
-		LogWarn(COMPONENT_DISPATCH,
-			"Cannot set udp socket for %s as non blocking, error %d(%s)",
-			tags[p], errno, strerror(errno));
+	if (udp_socket[p] != -1) {
+		/* We prefer using non-blocking socket
+		 * in the specific case */
+		if (fcntl(udp_socket[p], F_SETFL, FNDELAY) == -1) {
+			LogWarn(COMPONENT_DISPATCH,
+				"Cannot set udp socket for %s as non blocking, error %d(%s)",
+				tags[p], errno, strerror(errno));
 
-		return -1;
+			return -1;
+		}
 	}
 
 	return 0;
@@ -804,21 +846,24 @@ static int alloc_socket_setopts(int p)
  */
 static int Allocate_sockets_V4(int p)
 {
-	udp_socket[p] = socket(AF_INET,
-			       SOCK_DGRAM,
-			       IPPROTO_UDP);
+	udp_socket[p] = tcp_socket[p] = -1;
+	if (nfs_param.core_param.enable_UDP) {
+		udp_socket[p] = socket(AF_INET,
+				       SOCK_DGRAM,
+				       IPPROTO_UDP);
 
-	if (udp_socket[p] == -1) {
-		if (errno == EAFNOSUPPORT) {
-			LogInfo(COMPONENT_DISPATCH,
-				"No V6 and V4 intfs configured?!");
+		if (udp_socket[p] == -1) {
+			if (errno == EAFNOSUPPORT) {
+				LogInfo(COMPONENT_DISPATCH,
+					"No V6 and V4 intfs configured?!");
+			}
+
+			LogWarn(COMPONENT_DISPATCH,
+				"Cannot allocate a udp socket for %s, error %d(%s)",
+				tags[p], errno, strerror(errno));
+
+			return -1;
 		}
-
-		LogWarn(COMPONENT_DISPATCH,
-			"Cannot allocate a udp socket for %s, error %d(%s)",
-			tags[p], errno, strerror(errno));
-
-		return -1;
 	}
 
 	tcp_socket[p] = socket(AF_INET,
@@ -862,7 +907,7 @@ static int allocate_socket_vsock(void)
 	}
 
 	LogDebug(COMPONENT_DISPATCH,
-		"Socket numbers are: %s tcp=%u",
+		"Socket numbers are: %s tcp=%d",
 		tags[P_NFS_VSOCK],
 		tcp_socket[P_NFS_VSOCK]);
 	return 0;
@@ -882,37 +927,39 @@ static void Allocate_sockets(void)
 	for (p = P_NFS; p < P_COUNT; p++) {
 		/* Initialize all the sockets to -1 because
 		 * it makes some code later easier */
-		udp_socket[p] = -1;
-		tcp_socket[p] = -1;
+		udp_socket[p] = tcp_socket[p] = -1;
 
 		if (nfs_protocol_enabled(p)) {
 			if (v6disabled)
 				goto try_V4;
 
-			udp_socket[p] = socket(AF_INET6,
-					       SOCK_DGRAM,
-					       IPPROTO_UDP);
+			if (nfs_param.core_param.enable_UDP) {
+				udp_socket[p] = socket(AF_INET6,
+						       SOCK_DGRAM,
+						       IPPROTO_UDP);
 
-			if (udp_socket[p] == -1) {
-				/*
-				 * We assume that EAFNOSUPPORT points
-				 * to the likely case when the host has
-				 * V6 interfaces disabled. So we will
-				 * try to use the existing V4 interfaces
-				 * instead
-				 */
-				if (errno == EAFNOSUPPORT) {
-					v6disabled = true;
-					LogWarn(COMPONENT_DISPATCH,
-					    "System may not have V6 intfs configured error %d(%s)",
-					    errno, strerror(errno));
+				if (udp_socket[p] == -1) {
+					/*
+					 * We assume that EAFNOSUPPORT points
+					 * to the likely case when the host has
+					 * V6 interfaces disabled. So we will
+					 * try to use the existing V4 interfaces
+					 * instead
+					 */
+					if (errno == EAFNOSUPPORT) {
+						v6disabled = true;
+						LogWarn(COMPONENT_DISPATCH,
+						    "System may not have V6 intfs configured error %d(%s)",
+						    errno, strerror(errno));
 
-					goto try_V4;
+						goto try_V4;
+					}
+
+					LogFatal(COMPONENT_DISPATCH,
+						 "Cannot allocate a udp socket for %s, error %d(%s)",
+						 tags[p], errno,
+						 strerror(errno));
 				}
-
-				LogFatal(COMPONENT_DISPATCH,
-					 "Cannot allocate a udp socket for %s, error %d(%s)",
-					 tags[p], errno, strerror(errno));
 			}
 
 			tcp_socket[p] = socket(AF_INET6,
@@ -949,7 +996,7 @@ try_V4:
 					 p, tags[p]);
 			}
 			LogDebug(COMPONENT_DISPATCH,
-				"Socket numbers are: %s tcp=%u udp=%u",
+				"Socket numbers are: %s tcp=%d udp=%d",
 				tags[p],
 				tcp_socket[p],
 				udp_socket[p]);
@@ -992,26 +1039,30 @@ void Clean_RPC(void)
 static bool __Register_program(protos prot, int flag, int vers)
 {
 	if ((NFS_options & flag) != 0) {
-		LogInfo(COMPONENT_DISPATCH, "Registering %s V%d/UDP",
-			tags[prot], (int)vers);
-
-		/* XXXX fix svc_register! */
-		if (!UDP_REGISTER(prot, vers, netconfig_udpv4)) {
-			LogMajor(COMPONENT_DISPATCH,
-				 "Cannot register %s V%d on UDP", tags[prot],
-				 (int)vers);
-			return false;
-		}
-
-		if (!v6disabled && netconfig_udpv6) {
-			LogInfo(COMPONENT_DISPATCH, "Registering %s V%d/UDPv6",
+		if (nfs_param.core_param.enable_UDP) {
+			LogInfo(COMPONENT_DISPATCH, "Registering %s V%d/UDP",
 				tags[prot], (int)vers);
-			if (!UDP_REGISTER(prot, vers, netconfig_udpv6)) {
+
+			/* XXXX fix svc_register! */
+			if (!UDP_REGISTER(prot, vers, netconfig_udpv4)) {
 				LogMajor(COMPONENT_DISPATCH,
-					 "Cannot register %s V%d on UDPv6",
+					 "Cannot register %s V%d on UDP",
 					 tags[prot], (int)vers);
 				return false;
 			}
+
+			if (!v6disabled && netconfig_udpv6) {
+				LogInfo(COMPONENT_DISPATCH,
+					"Registering %s V%d/UDPv6",
+					tags[prot], (int)vers);
+				if (!UDP_REGISTER(prot, vers,
+						  netconfig_udpv6)) {
+					LogMajor(COMPONENT_DISPATCH,
+						 "Cannot register %s V%d on UDPv6",
+						 tags[prot], (int)vers);
+					return false;
+				}
+			}
 		}
 
 #ifndef _NO_TCP_REGISTER
@@ -1080,7 +1131,8 @@ void nfs_Init_svc(void)
 
 	/* New TI-RPC package init function */
 	svc_params.disconnect_cb = NULL;
-	svc_params.request_cb = nfs_rpc_decode_request;
+	svc_params.alloc_cb = alloc_nfs_request;
+	svc_params.free_cb = free_nfs_request;
 	svc_params.flags = SVC_INIT_EPOLL;	/* use EPOLL event mgmt */
 	svc_params.flags |= SVC_INIT_NOREG_XPRTS; /* don't call xprt_register */
 	svc_params.max_connections = nfs_param.core_param.rpc.max_connections;
@@ -1136,7 +1188,7 @@ void nfs_Init_svc(void)
 	netconfig_udpv6 = (struct netconfig *)getnetconfigent("udp6");
 	if (netconfig_udpv6 == NULL)
 		LogInfo(COMPONENT_DISPATCH,
-			 "Cannot get udp6 netconfig, cannot get an entry for udp6 in netconfig file. Check file /etc/netconfig...");
+			"Cannot get udp6 netconfig, cannot get an entry for udp6 in netconfig file. Check file /etc/netconfig...");
 
 	/* Get the netconfig entries from /etc/netconfig */
 	netconfig_tcpv6 = (struct netconfig *)getnetconfigent("tcp6");
@@ -1247,108 +1299,73 @@ static enum xprt_stat nfs_rpc_tcp_user_data(SVCXPRT *newxprt)
 static enum xprt_stat nfs_rpc_free_user_data(SVCXPRT *xprt)
 {
 	if (xprt->xp_u2) {
-		nfs_dupreq_put_drc(xprt->xp_u2, DRC_FLAG_RELEASE);
+		nfs_dupreq_put_drc(xprt->xp_u2);
 		xprt->xp_u2 = NULL;
 	}
 	return XPRT_DESTROYED;
 }
 
-
 /**
  * @brief Allocate a new request
  *
  * @param[in] xprt Transport to use
+ * @param[in] xdrs XDR to use
  *
- * @return New request data
+ * @return New svc request
  */
-static inline request_data_t *alloc_nfs_request(SVCXPRT *xprt, XDR *xdrs)
+static struct svc_req *alloc_nfs_request(SVCXPRT *xprt, XDR *xdrs)
 {
-	request_data_t *reqdata = pool_alloc(nfs_request_pool);
-
-	(void) atomic_inc_uint64_t(&nfs_health_.enqueued_reqs);
-
-	/* set the request as NFS already-read */
-	reqdata->rtype = NFS_REQUEST;
-
-	/* set up req */
-	SVC_REF(xprt, SVC_REF_FLAG_NONE);
-	reqdata->r_u.req.svc.rq_xprt = xprt;
-	reqdata->r_u.req.svc.rq_xdrs = xdrs;
-	reqdata->r_u.req.svc.rq_refcnt = 1;
-	return reqdata;
-}
-
-int free_nfs_request(request_data_t *reqdata)
-{
-	SVCXPRT *xprt = reqdata->r_u.req.svc.rq_xprt;
-	uint32_t refs = atomic_dec_uint32_t(&reqdata->r_u.req.svc.rq_refcnt);
-
-	LogDebug(COMPONENT_DISPATCH,
-		 "%s: %p fd %d xp_refcnt %" PRIu32 " rq_refcnt %" PRIu32,
-		 __func__,
-		 xprt, xprt->xp_fd, xprt->xp_refcnt,
-		 refs);
-
-	if (refs)
-		return refs;
-
-	switch (reqdata->rtype) {
-	case NFS_REQUEST:
-		/* dispose RPC header */
-		if (reqdata->r_u.req.svc.rq_auth)
-			SVCAUTH_RELEASE(&(reqdata->r_u.req.svc));
-		XDR_DESTROY(reqdata->r_u.req.svc.rq_xdrs);
-		break;
-	default:
-		break;
-	}
-	SVC_RELEASE(xprt, SVC_RELEASE_FLAG_NONE);
-	pool_free(nfs_request_pool, reqdata);
-	(void) atomic_inc_uint64_t(&nfs_health_.dequeued_reqs);
-	return 0;
-}
-
-static enum xprt_stat nfs_rpc_decode_request(SVCXPRT *xprt, XDR *xdrs)
-{
-	request_data_t *reqdata;
-	enum xprt_stat stat;
+	nfs_request_t *reqdata = gsh_calloc(1, sizeof(nfs_request_t));
 
 	if (!xprt) {
 		LogFatal(COMPONENT_DISPATCH,
 			 "missing xprt!");
-		return XPRT_DIED;
 	}
+
 	if (!xdrs) {
 		LogFatal(COMPONENT_DISPATCH,
 			 "missing xdrs!");
-		return XPRT_DIED;
 	}
+
 	LogDebug(COMPONENT_DISPATCH,
 		 "%p fd %d context %p",
 		 xprt, xprt->xp_fd, xdrs);
 
-	reqdata = alloc_nfs_request(xprt, xdrs);
+	(void) atomic_inc_uint64_t(&nfs_health_.enqueued_reqs);
+
+	/* set up req */
+	SVC_REF(xprt, SVC_REF_FLAG_NONE);
+	reqdata->svc.rq_xprt = xprt;
+	reqdata->svc.rq_xdrs = xdrs;
+	reqdata->svc.rq_refcnt = 1;
+
 #if HAVE_BLKIN
-	blkin_init_new_trace(&reqdata->r_u.req.svc.bl_trace, "nfs-ganesha",
+	blkin_init_new_trace(&reqdata->svc.bl_trace, "nfs-ganesha",
 			&xprt->blkin.endp);
 #endif
 
 #if defined(HAVE_BLKIN)
 	BLKIN_TIMESTAMP(
-		&reqdata->r_u.req.svc.bl_trace, &xprt->blkin.endp, "pre-recv");
+		&reqdata->svc.bl_trace, &xprt->blkin.endp, "pre-recv");
 #endif
 
-	stat = SVC_DECODE(&reqdata->r_u.req.svc);
+	return &reqdata->svc;
+}
+
+static void free_nfs_request(struct svc_req *req, enum xprt_stat stat)
+{
+	nfs_request_t *reqdata = container_of(req, nfs_request_t, svc);
+	SVCXPRT *xprt = reqdata->svc.rq_xprt;
 
 #if defined(HAVE_BLKIN)
 	BLKIN_TIMESTAMP(
-		&reqdata->r_u.req.svc.bl_trace, &xprt->blkin.endp, "post-recv");
+		&reqdata->svc.bl_trace, &xprt->blkin.endp, "post-recv");
 
 	BLKIN_KEYVAL_INTEGER(
-		&reqdata->r_u.req.svc.bl_trace,
-		&reqdata->r_u.req.xprt->blkin.endp,
+		&reqdata->svc.bl_trace,
+		&reqdata->xprt->blkin.endp,
 		"rq-xid",
-		reqdata->r_u.req.svc.rq_xid);
+		reqdata->svc.rq_xid);
 #endif
 
 	if (unlikely(stat > XPRT_DESTROYED)) {
@@ -1356,29 +1373,33 @@ static enum xprt_stat nfs_rpc_decode_request(SVCXPRT *xprt, XDR *xdrs)
 			"SVC_DECODE on %p fd %d returned unknown %u",
 			xprt, xprt->xp_fd,
 			stat);
-	} else {
+	} else if (isDebug(COMPONENT_DISPATCH)) {
 		sockaddr_t addr;
-		char addrbuf[SOCK_NAME_MAX + 1];
+		char addrbuf[SOCK_NAME_MAX];
+		struct display_buffer dspbuf = {
+					sizeof(addrbuf), addrbuf, addrbuf};
 
-		if (isDebug(COMPONENT_DISPATCH)) {
-			if (copy_xprt_addr(&addr, xprt) == 1)
-				sprint_sockaddr(&addr, addrbuf,
-						sizeof(addrbuf));
-			else
-				sprintf(addrbuf, "<unresolved>");
-		}
+		if (copy_xprt_addr(&addr, xprt) == 1)
+			display_sockaddr(&dspbuf, &addr);
+		else
+			display_cat(&dspbuf, "<unresolved>");
 
 		LogDebug(COMPONENT_DISPATCH,
 			 "SVC_DECODE on %p fd %d (%s) xid=%" PRIu32
 			 " returned %s",
 			 xprt, xprt->xp_fd,
 			 addrbuf,
-			 reqdata->r_u.req.svc.rq_msg.rm_xid,
+			 reqdata->svc.rq_msg.rm_xid,
 			 xprt_stat_s[stat]);
 	}
 
-	/* refresh status before possible release */
-	stat = SVC_STAT(xprt);
-	free_nfs_request(reqdata);
-	return stat;
+	LogFullDebug(COMPONENT_DISPATCH,
+		     "%s: %p fd %d xp_refcnt %" PRIu32,
+		     __func__, xprt, xprt->xp_fd, xprt->xp_refcnt);
+
+	gsh_free(reqdata);
+
+	SVC_RELEASE(xprt, SVC_REF_FLAG_NONE);
+
+	(void) atomic_inc_uint64_t(&nfs_health_.dequeued_reqs);
 }
