@@ -103,7 +103,8 @@ int _9p_mknod(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 		return _9p_rerror(req9p, msgtag, ENAMETOOLONG, plenout,
 				  preply);
 	}
-	snprintf(obj_name, sizeof(obj_name), "%.*s", *name_len, name_str);
+
+	_9p_get_fname(obj_name, *name_len, name_str);
 
 	/* Set the nodetype */
 	if (S_ISDIR(*mode))
