@@ -43,8 +43,8 @@
 #include "export_mgr.h"
 
 struct proc_state {
-	exports head;
-	exports tail;
+	mnt3_exports head;
+	mnt3_exports tail;
 	int retval;
 };
 
@@ -201,6 +201,7 @@ void mnt_Export_Free(nfs_res_t *res)
 			gsh_free(grp);
 			grp = next_grp;
 		}
+		gsh_free(exp->ex_dir);
 		gsh_free(exp);
 		exp = next_exp;
 	}
