@@ -129,7 +129,7 @@ int mnt_Mnt(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	export_check_access();
 
 	if ((op_ctx->export_perms->options & EXPORT_OPTION_NFSV3) == 0) {
-		LogInfo(COMPONENT_NFSPROTO,
+		LogInfoAlt(COMPONENT_NFSPROTO, COMPONENT_EXPORT,
 			"MOUNT: Export entry %s does not support NFS v3 for client %s",
 			export_path(export),
 			op_ctx->client
@@ -139,6 +139,17 @@ int mnt_Mnt(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 		goto out;
 	}
 
+	if ((op_ctx->export_perms->options & EXPORT_OPTION_ACCESS_MASK) == 0) {
+		LogInfoAlt(COMPONENT_NFSPROTO, COMPONENT_EXPORT,
+			"MOUNT: Export entry %s does not allow access for client %s",
+			export_path(export),
+			op_ctx->client
+				? op_ctx->client->hostaddr_str
+				: "unknown client");
+		res->res_mnt3.fhs_status = MNT3ERR_ACCES;
+		goto out;
+	}
+
 	/* retrieve the associated NFS handle */
 	if (arg->arg_mnt[0] != '/' ||
 	    !strcmp(arg->arg_mnt, export_path(export))) {
