@@ -131,18 +131,8 @@ int nfs3_readdir(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 					EXPORT_OPTION_USE_COOKIE_VERIFIER);
 	READDIR3resfail *resfail = &res->res_readdir3.READDIR3res_u.resfail;
 
-	if (isDebug(COMPONENT_NFSPROTO) || isDebug(COMPONENT_NFS_READDIR)) {
-		char str[LEN_FH_STR];
-
-		nfs_FhandleToStr(req->rq_msg.cb_vers,
-				 &(arg->arg_readdir3.dir),
-				 NULL,
-				 str);
-
-		LogDebugAlt(COMPONENT_NFSPROTO, COMPONENT_NFS_READDIR,
-			    "REQUEST PROCESSING: Calling nfs_Readdir handle: %s",
-			    str);
-	}
+	LogNFS3_Operation(COMPONENT_NFSPROTO, req, &arg->arg_readdir3.dir,
+			  "");
 
 	READDIR3resok * const RES_READDIR3_OK =
 	    &res->res_readdir3.READDIR3res_u.resok;
@@ -188,15 +178,12 @@ int nfs3_readdir(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	}
 
 	/* To make or check the cookie verifier */
-	memset(cookie_verifier, 0, sizeof(cookieverf3));
+	memset(cookie_verifier, 0, sizeof(cookie_verifier));
 
-	/* If cookie verifier is used, then a
-	 * non-trivial value is returned to the
-	 * client.
-	 *
-	 * This value is the ctime of the directory. If verifier is
-	 * unused (as in many NFS Servers) then only a set of zeros
-	 * is returned (trivial value).
+	/* If cookie verifier is used, then an non-trivial value is
+	 * returned to the client This value is the change attribute of the
+	 * directory. If verifier is unused (as in many NFS Servers) then
+	 * only a set of zeros is returned (trivial value)
 	 */
 	if (use_cookie_verifier) {
 		struct attrlist attrs;
@@ -214,9 +201,8 @@ int nfs3_readdir(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 			goto out;
 		}
 
-		memcpy(cookie_verifier,
-		       &attrs.ctime.tv_sec,
-		       sizeof(attrs.ctime.tv_sec));
+		memcpy(cookie_verifier, &attrs.change,
+		       MIN(sizeof(cookie_verifier), sizeof(attrs.change)));
 
 		/* Done with the attrs */
 		fsal_release_attrs(&attrs);
