@@ -57,8 +57,9 @@
  *
  */
 
-int nfs4_op_layoutcommit(struct nfs_argop4 *op, compound_data_t *data,
-			 struct nfs_resop4 *resp)
+enum nfs_req_result nfs4_op_layoutcommit(struct nfs_argop4 *op,
+					 compound_data_t *data,
+					 struct nfs_resop4 *resp)
 {
 	/* Convenience alias for arguments */
 	LAYOUTCOMMIT4args * const args = &op->nfs_argop4_u.oplayoutcommit;
@@ -91,14 +92,14 @@ int nfs4_op_layoutcommit(struct nfs_argop4 *op, compound_data_t *data,
 
 	if (data->minorversion == 0) {
 		res_LAYOUTCOMMIT4->locr_status = NFS4ERR_INVAL;
-		return res_LAYOUTCOMMIT4->locr_status;
+		return NFS_REQ_ERROR;
 	}
 
 	nfs_status = nfs4_sanity_check_FH(data, REGULAR_FILE, false);
 
 	if (nfs_status != NFS4_OK) {
 		res_LAYOUTCOMMIT4->locr_status = nfs_status;
-		return res_LAYOUTCOMMIT4->locr_status;
+		return NFS_REQ_ERROR;
 	}
 
 
@@ -146,7 +147,7 @@ int nfs4_op_layoutcommit(struct nfs_argop4 *op, compound_data_t *data,
 
 	arg.type = layout_state->state_data.layout.state_layout_type;
 
-	PTHREAD_RWLOCK_wrlock(&data->current_obj->state_hdl->state_lock);
+	STATELOCK_lock(data->current_obj);
 	glist_for_each(glist, &layout_state->state_data.layout.state_segments) {
 		segment = glist_entry(glist,
 				      state_layout_segment_t,
@@ -163,8 +164,7 @@ int nfs4_op_layoutcommit(struct nfs_argop4 *op, compound_data_t *data,
 						&res);
 
 		if (nfs_status != NFS4_OK) {
-			PTHREAD_RWLOCK_unlock(
-				&data->current_obj->state_hdl->state_lock);
+			STATELOCK_unlock(data->current_obj);
 			goto out;
 		}
 
@@ -176,7 +176,7 @@ int nfs4_op_layoutcommit(struct nfs_argop4 *op, compound_data_t *data,
 		xdr_setpos(&lou_body, beginning);
 	}
 
-	PTHREAD_RWLOCK_unlock(&data->current_obj->state_hdl->state_lock);
+	STATELOCK_unlock(data->current_obj);
 
 	resok->locr_newsize.ns_sizechanged = res.size_supplied;
 
@@ -195,7 +195,7 @@ int nfs4_op_layoutcommit(struct nfs_argop4 *op, compound_data_t *data,
 
 	res_LAYOUTCOMMIT4->locr_status = nfs_status;
 
-	return res_LAYOUTCOMMIT4->locr_status;
+	return nfsstat4_to_nfs_req_result(res_LAYOUTCOMMIT4->locr_status);
 }				/* nfs41_op_layoutcommit */
 
 /**
