@@ -65,8 +65,9 @@
  * @see nfs4_Compound
  */
 
-int nfs4_op_layoutreturn(struct nfs_argop4 *op, compound_data_t *data,
-			 struct nfs_resop4 *resp)
+enum nfs_req_result nfs4_op_layoutreturn(struct nfs_argop4 *op,
+					 compound_data_t *data,
+					 struct nfs_resop4 *resp)
 {
 	/* Convenience alias for arguments */
 	LAYOUTRETURN4args * const arg_LAYOUTRETURN4 =
@@ -118,7 +119,7 @@ int nfs4_op_layoutreturn(struct nfs_argop4 *op, compound_data_t *data,
 
 	if (data->minorversion == 0) {
 		res_LAYOUTRETURN4->lorr_status = NFS4ERR_INVAL;
-		return res_LAYOUTRETURN4->lorr_status;
+		return NFS_REQ_ERROR;
 	}
 
 	switch (arg_LAYOUTRETURN4->lora_layoutreturn.lr_returntype) {
@@ -152,8 +153,7 @@ int nfs4_op_layoutreturn(struct nfs_argop4 *op, compound_data_t *data,
 		spec.offset = lr_layout->lrf_offset;
 		spec.length = lr_layout->lrf_length;
 
-		PTHREAD_RWLOCK_wrlock(
-			&data->current_obj->state_hdl->state_lock);
+		STATELOCK_lock(data->current_obj);
 
 		res_LAYOUTRETURN4->lorr_status = nfs4_return_one_state(
 			data->current_obj,
@@ -167,8 +167,7 @@ int nfs4_op_layoutreturn(struct nfs_argop4 *op, compound_data_t *data,
 			lr_layout->lrf_body.lrf_body_val,
 			&deleted);
 
-		PTHREAD_RWLOCK_unlock(
-			&data->current_obj->state_hdl->state_lock);
+		STATELOCK_unlock(data->current_obj);
 
 		if (res_LAYOUTRETURN4->lorr_status == NFS4_OK) {
 			if (deleted) {
@@ -308,7 +307,7 @@ int nfs4_op_layoutreturn(struct nfs_argop4 *op, compound_data_t *data,
 				}
 			}
 
-			PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
+			STATELOCK_lock(obj);
 
 			res_LAYOUTRETURN4->lorr_status = nfs4_return_one_state(
 			    obj,
@@ -321,7 +320,7 @@ int nfs4_op_layoutreturn(struct nfs_argop4 *op, compound_data_t *data,
 			    NULL,
 			    &deleted);
 
-			PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
+			STATELOCK_unlock(obj);
 
 			/* Release the state_t reference */
 			dec_state_t_ref(layout_state);
@@ -369,7 +368,7 @@ int nfs4_op_layoutreturn(struct nfs_argop4 *op, compound_data_t *data,
 		put_gsh_export(export);
 	}
 
-	return res_LAYOUTRETURN4->lorr_status;
+	return nfsstat4_to_nfs_req_result(res_LAYOUTRETURN4->lorr_status);
 }				/* nfs41_op_layoutreturn */
 
 /**
