@@ -57,8 +57,8 @@
  * @see nfs4_Compound
  */
 
-int nfs4_op_lockt(struct nfs_argop4 *op, compound_data_t *data,
-		  struct nfs_resop4 *resp)
+enum nfs_req_result nfs4_op_lockt(struct nfs_argop4 *op, compound_data_t *data,
+				  struct nfs_resop4 *resp)
 {
 	/* Alias for arguments */
 	LOCKT4args * const arg_LOCKT4 = &op->nfs_argop4_u.oplockt;
@@ -95,18 +95,18 @@ int nfs4_op_lockt(struct nfs_argop4 *op, compound_data_t *data,
 	res_LOCKT4->status = nfs4_sanity_check_FH(data, REGULAR_FILE, false);
 
 	if (res_LOCKT4->status != NFS4_OK)
-		return res_LOCKT4->status;
+		return NFS_REQ_ERROR;
 
 
 	/* Lock length should not be 0 */
 	if (arg_LOCKT4->length == 0LL) {
 		res_LOCKT4->status = NFS4ERR_INVAL;
-		return res_LOCKT4->status;
+		return NFS_REQ_ERROR;
 	}
 
-	if (nfs_in_grace()) {
+	if (!nfs_get_grace_status(false)) {
 		res_LOCKT4->status = NFS4ERR_GRACE;
-		return res_LOCKT4->status;
+		return NFS_REQ_ERROR;
 	}
 
 	/* Convert lock parameters to internal types */
@@ -124,7 +124,7 @@ int nfs4_op_lockt(struct nfs_argop4 *op, compound_data_t *data,
 		LogDebug(COMPONENT_NFS_V4_LOCK,
 			 "Invalid lock type");
 		res_LOCKT4->status = NFS4ERR_INVAL;
-		return res_LOCKT4->status;
+		goto out;
 	}
 
 	lock_desc.lock_start = arg_LOCKT4->offset;
@@ -146,7 +146,7 @@ int nfs4_op_lockt(struct nfs_argop4 *op, compound_data_t *data,
 			 "LOCK failed length overflow start %"PRIx64
 			 " length %"PRIx64,
 			 lock_desc.lock_start, lock_desc.lock_length);
-		return res_LOCKT4->status;
+		goto out;
 	}
 
 	/* Check for range overflow past maxfilesize.  Comparing beyond 2^64 is
@@ -172,16 +172,15 @@ int nfs4_op_lockt(struct nfs_argop4 *op, compound_data_t *data,
 
 	if (rc != CLIENT_ID_SUCCESS) {
 		res_LOCKT4->status = clientid_error_to_nfsstat(rc);
-		return res_LOCKT4->status;
+		goto out;
 	}
 
 	PTHREAD_MUTEX_lock(&clientid->cid_mutex);
 
 	if (data->minorversion == 0 && !reserve_lease(clientid)) {
 		PTHREAD_MUTEX_unlock(&clientid->cid_mutex);
-		dec_client_id_ref(clientid);
 		res_LOCKT4->status = NFS4ERR_EXPIRED;
-		return res_LOCKT4->status;
+		goto out_clientid;
 	}
 
 	PTHREAD_MUTEX_unlock(&clientid->cid_mutex);
@@ -204,7 +203,7 @@ int nfs4_op_lockt(struct nfs_argop4 *op, compound_data_t *data,
 		LogEvent(COMPONENT_NFS_V4_LOCK,
 			 "LOCKT unable to create lock owner");
 		res_LOCKT4->status = NFS4ERR_SERVERFAULT;
-		goto out;
+		goto out_clientid;
 	}
 
 	LogLock(COMPONENT_NFS_V4_LOCK, NIV_FULL_DEBUG, "LOCKT",
@@ -258,18 +257,17 @@ int nfs4_op_lockt(struct nfs_argop4 *op, compound_data_t *data,
 	if (state != NULL)
 		dec_state_t_ref(state);
 
- out:
-
+out_clientid:
 	/* Update the lease before exit */
 	if (data->minorversion == 0) {
 		PTHREAD_MUTEX_lock(&clientid->cid_mutex);
 		update_lease(clientid);
 		PTHREAD_MUTEX_unlock(&clientid->cid_mutex);
 	}
-
 	dec_client_id_ref(clientid);
-
-	return res_LOCKT4->status;
+out:
+	nfs_put_grace_status();
+	return nfsstat4_to_nfs_req_result(res_LOCKT4->status);
 }				/* nfs4_op_lockt */
 
 /**
