@@ -186,10 +186,12 @@ void nfs_put_grace_status(void)
 {
 	uint32_t cur;
 
-	cur = __sync_fetch_and_sub(&grace_status, GRACE_STATUS_REF_INCREMENT);
+	cur = __sync_sub_and_fetch(&grace_status, GRACE_STATUS_REF_INCREMENT);
 	if (cur & GRACE_STATUS_CHANGE_REQ &&
-	    !(cur >> GRACE_STATUS_COUNTER_SHIFT))
+	    !(cur >> GRACE_STATUS_COUNTER_SHIFT)) {
+		nfs_notify_grace_norefs_waiters();
 		reaper_wake();
+	}
 }
 
 /**
@@ -232,10 +234,11 @@ static void nfs4_set_enforcing(void)
  * will be passed to this routine inside of the grace start structure.
  *
  * @param[in] gsp Grace period start information
+ * Returns 0 on success, -EAGAIN on failure to enforce grace.
  */
-void nfs_start_grace(nfs_grace_start_t *gsp)
+int nfs_start_grace(nfs_grace_start_t *gsp)
 {
-	int ret;
+	int ret = 0;
 	bool was_grace;
 	uint32_t cur, old, pro;
 
@@ -300,9 +303,15 @@ void nfs_start_grace(nfs_grace_start_t *gsp)
 	/*
 	 * If we were not in a grace period before and there were still
 	 * references outstanding, then we can't do anything else.
+	 * Fail with -EAGAIN so that caller can retry if needed.
 	 */
-	if (!was_grace && (old & GRACE_STATUS_COUNT_MASK))
+	if (!was_grace && (old & GRACE_STATUS_COUNT_MASK)) {
+		LogEvent(COMPONENT_STATE,
+			 "Unable to start grace, grace status 0x%x",
+			 grace_status);
+		ret = -EAGAIN;
 		goto out;
+	}
 
 	__sync_synchronize();
 
@@ -327,6 +336,7 @@ void nfs_start_grace(nfs_grace_start_t *gsp)
 	 * existing grace period.
 	 */
 	if (!gsp && !was_grace) {
+		nfs4_cleanup_clid_entries();
 		nfs4_recovery_load_clids(NULL);
 	} else if (gsp && gsp->event != EVENT_JUST_GRACE) {
 		/*
@@ -345,15 +355,24 @@ void nfs_start_grace(nfs_grace_start_t *gsp)
 			if (gsp->event == EVENT_RELEASE_IP) {
 				PTHREAD_MUTEX_unlock(&grace_mutex);
 				nfs_release_v4_clients(gsp->ipaddr);
-				return;
+				return ret;
 			}
 			else {
+				/*
+				 * If we're already in a grace period,
+				 * it can not break the existing count,
+				 * other case, which should not be affected
+				 * by the last count, should be cleanup.
+				 */
+				if (!was_grace)
+					nfs4_cleanup_clid_entries();
 				nfs4_recovery_load_clids(gsp);
 			}
 		}
 	}
 out:
 	PTHREAD_MUTEX_unlock(&grace_mutex);
+	return ret;
 }
 
 /**
@@ -563,6 +582,26 @@ void nfs_notify_grace_waiters(void)
 	pthread_mutex_unlock(&enforcing_mutex);
 }
 
+static pthread_cond_t norefs_cond = PTHREAD_COND_INITIALIZER;
+static pthread_mutex_t norefs_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+void nfs_wait_for_grace_norefs(void)
+{
+	pthread_mutex_lock(&norefs_mutex);
+	struct timespec	timeo = { .tv_sec = time(NULL) + 5,
+				  .tv_nsec = 0 };
+	pthread_cond_timedwait(&norefs_cond, &norefs_mutex,
+			       &timeo);
+	pthread_mutex_unlock(&norefs_mutex);
+}
+
+void nfs_notify_grace_norefs_waiters(void)
+{
+	pthread_mutex_lock(&norefs_mutex);
+	pthread_cond_broadcast(&norefs_cond);
+	pthread_mutex_unlock(&norefs_mutex);
+}
+
 /**
  * @brief Create an entry in the recovery directory
  *
@@ -620,7 +659,7 @@ void  nfs4_chk_clid_impl(nfs_client_id_t *clientid, clid_entry_t **clid_ent_arg)
 	clid_entry_t *clid_ent;
 	*clid_ent_arg = NULL;
 
-	LogDebug(COMPONENT_CLIENTID, "chk for %lu",
+	LogDebug(COMPONENT_CLIENTID, "chk for %" PRIu64,
 		 clientid->cid_clientid);
 
 	/* If there were no clients at time of restart, we're done */
@@ -674,9 +713,6 @@ static void nfs4_recovery_load_clids(nfs_grace_start_t *gsp)
 {
 	LogDebug(COMPONENT_STATE, "Load recovery cli %p", gsp);
 
-	/* A NULL gsp pointer indicates an initial startup grace period */
-	if (gsp == NULL)
-		nfs4_cleanup_clid_entries();
 	recovery_backend->recovery_read_clids(gsp, nfs4_add_clid_entry,
 						nfs4_add_rfh_entry);
 }
@@ -958,6 +994,9 @@ static void nfs_release_nlm_state(char *release_ip)
 	char serverip[SOCK_NAME_MAX];
 	int i;
 
+	if (!nfs_param.core_param.enable_NLM)
+		return;
+
 	LogDebug(COMPONENT_STATE, "Release all NLM locks");
 
 	cancel_all_nlm_blocked();
