@@ -34,6 +34,8 @@
 #include <boost/filesystem/exception.hpp>
 #include <boost/program_options.hpp>
 
+#include "gtest.hh"
+
 extern "C" {
 /* Manually forward this, as 9P is not C++ safe */
 void admin_halt(void);
@@ -47,8 +49,6 @@ void admin_halt(void);
 #include "../FSAL/Stackable_FSALs/FSAL_MDCACHE/mdcache_debug.h"
 }
 
-#include "gtest.hh"
-
 #define TEST_ROOT "readdir_latency"
 #define TEST_DIR "test_directory"
 #define DIR_COUNT 100000
@@ -124,7 +124,7 @@ namespace {
   }
 
   struct cb_data {
-        u8 *cursor;
+        uint8_t *cursor;
         unsigned int count;
         unsigned int max;
   };
@@ -161,7 +161,7 @@ TEST_F(ReaddirEmptyLatencyTest, FSALREADDIR)
   uint64_t cookie = 0LL;
   unsigned int num_entries = 0;
   bool eod_met = false;
-  struct cb_data tracker = { NULL };
+  struct cb_data tracker = { 0 };
   struct timespec s_time, e_time;
 
   now(&s_time);
