@@ -42,6 +42,7 @@
 #include <libgen.h>
 #include <sys/resource.h>
 #include <execinfo.h>
+#include <assert.h>
 
 #include "log.h"
 #include "gsh_list.h"
@@ -381,27 +382,30 @@ void SetNamePgm(const char *nom)
 
 	/* This function isn't thread-safe because the name of the program
 	 * is common among all the threads. */
-	if (strmaxcpy(program_name, nom, sizeof(program_name)) == -1)
+	if (strlcpy(program_name, nom, sizeof(program_name))
+	    >= sizeof(program_name))
 		LogFatal(COMPONENT_LOG, "Program name %s too long", nom);
-}				/* SetNamePgm */
+}
 
 /*
  * Set the hostname.
  */
 void SetNameHost(const char *name)
 {
-	if (strmaxcpy(hostname, name, sizeof(hostname)) == -1)
+	if (strlcpy(hostname, name, sizeof(hostname))
+	    >= sizeof(hostname))
 		LogFatal(COMPONENT_LOG, "Host name %s too long", name);
-}				/* SetNameHost */
+}
 
 /* Set the function name in progress. */
 void SetNameFunction(const char *nom)
 {
-	strlcpy(thread_name, nom, sizeof(thread_name));
-	if (strlen(nom) >= sizeof(thread_name))
+	if (strlcpy(thread_name, nom, sizeof(thread_name))
+	    >= sizeof(thread_name)) {
 		LogWarn(COMPONENT_LOG,
 			"Thread name %s too long truncated to %s",
 			nom, thread_name);
+	}
 	clientip = NULL;
 }
 
@@ -750,8 +754,7 @@ int create_log_facility(const char *name, lf_function_t *log_func,
 				 "New log file path empty or too long");
 			return -EINVAL;
 		}
-		dir = alloca(strlen(private) + 1);
-		strcpy(dir, private);
+		dir = gsh_strdupa(private);
 		dir = dirname(dir);
 		rc = access(dir, W_OK);
 		if (rc != 0) {
@@ -1020,8 +1023,7 @@ int set_log_destination(const char *name, char *dest)
 	if (facility->lf_func == log_to_file) {
 		char *logfile, *dir;
 
-		dir = alloca(strlen(dest) + 1);
-		strcpy(dir, dest);
+		dir = gsh_strdupa(dest);
 		dir = dirname(dir);
 		rc = access(dir, W_OK);
 		if (rc != 0) {
@@ -2541,9 +2543,16 @@ static int log_conf_commit(void *node, void *link_mem, void *self_struct,
 		if (logger->comp_log_level != NULL) {
 			LogEvent(COMPONENT_CONFIG,
 				 "Switching to new component log levels");
-			if (component_log_level != default_log_levels)
-				gsh_free(component_log_level);
-			component_log_level = logger->comp_log_level;
+			if (component_log_level == default_log_levels) {
+				/* First time change from default log level */
+				component_log_level = logger->comp_log_level;
+				logger->comp_log_level = NULL;
+			} else {
+				/* update exisiting component_log_level */
+				memcpy(component_log_level,
+				  logger->comp_log_level,
+				  (sizeof(log_levels_t) * COMPONENT_COUNT));
+			}
 		}
 		ntirpc_pp.debug_flags = logger->rpc_debug_flags;
 		SetNTIRPCLogLevel(component_log_level[COMPONENT_TIRPC]);
@@ -2557,9 +2566,9 @@ static int log_conf_commit(void *node, void *link_mem, void *self_struct,
 				gsh_free(lf->user_time_fmt);
 			gsh_free(lf);
 		}
-		if (logger->comp_log_level != NULL)
-			gsh_free(logger->comp_log_level);
 	}
+	if (logger->comp_log_level != NULL)
+		gsh_free(logger->comp_log_level);
 	logger->logfields = NULL;
 	logger->comp_log_level = NULL;
 	return errcnt;
@@ -2663,3 +2672,36 @@ void gsh_backtrace(void)
 	}
 	PTHREAD_RWLOCK_unlock(&log_rwlock);
 }
+
+bool _ratelimit(struct ratelimit_state *rs, int *missed)
+{
+	bool ret;
+	time_t now;
+
+	/* If we fail to acquire the mutex, then we are alreday busy,
+	 * so don't log message (aka return false)
+	 */
+	if (pthread_mutex_trylock(&rs->mutex))
+		return false;
+
+	now = time(NULL);
+	if (now > rs->begin + rs->interval) { /* new interval */
+		*missed = rs->missed;
+		rs->begin = now;
+		rs->printed = 0;
+		rs->missed = 0;
+	} else {
+		*missed = 0;
+	}
+
+	if (rs->burst > rs->printed) {
+		rs->printed++;
+		ret = true;
+	} else {
+		rs->missed++;
+		ret = false;
+	}
+	(void)pthread_mutex_unlock(&rs->mutex);
+
+	return ret;
+}
