@@ -20,8 +20,8 @@
 #-*- coding: utf-8 -*-
 
 import sys, time
-from PyQt4.QtCore import *
-from PyQt4 import QtDBus, QtGui
+from PyQt5.QtCore import *
+from PyQt5 import QtDBus, QtGui
 from collections import namedtuple
 
 Export = namedtuple('Export',
@@ -43,7 +43,7 @@ class ExportMgr(QtDBus.QDBusAbstractInterface):
     '''
     show_exports = pyqtSignal(tuple, list)
     display_export = pyqtSignal(int, str, str, str)
-    
+
     def __init__(self, service, path, connection,
                  show_status, parent=None):
         super(ExportMgr, self).__init__(service,
@@ -54,28 +54,28 @@ class ExportMgr(QtDBus.QDBusAbstractInterface):
         self.show_status = show_status
 
     def AddExport(self, conf_path, exp_expr):
-        async = self.asyncCall("AddExport", conf_path, exp_expr)
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("AddExport", conf_path, exp_expr)
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.exportadd_done)
 
     def UpdateExport(self, conf_path, exp_expr):
-        async = self.asyncCall("UpdateExport", conf_path, exp_expr)
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("UpdateExport", conf_path, exp_expr)
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.exportadd_done)
 
     def RemoveExport(self, exp_id):
-        async = self.asyncCall("RemoveExport", int(exp_id))
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("RemoveExport", int(exp_id))
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.exportrm_done)
 
     def DisplayExport(self, exp_id):
-        async = self.asyncCall("DisplayExport", int(exp_id))
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("DisplayExport", int(exp_id))
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.exportdisplay_done)
-        
+
     def ShowExports(self):
-        async = self.asyncCall("ShowExports")
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("ShowExports")
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.exportshow_done)
 
     def exportadd_done(self, call):
@@ -101,38 +101,38 @@ class ExportMgr(QtDBus.QDBusAbstractInterface):
             self.show_status.emit(False,
                                   "Error:" + str(reply.error().message()))
         else:
-            id = reply.argumentAt(0).toPyObject()
+            id_ = reply.argumentAt(0).toPyObject()
             fullpath = reply.argumentAt(1).toPyObject()
             pseudopath = reply.argumentAt(2).toPyObject()
             tag = reply.argumentAt(3).toPyObject()
-            self.display_export.emit(id, fullpath, pseudopath, tag)
-            
+            self.display_export.emit(id_, fullpath, pseudopath, tag)
+
     def exportshow_done(self, call):
         reply = QtDBus.QDBusPendingReply(call)
         if reply.isError():
             self.show_status.emit(False,
                                   "DBUS error:" + str(reply.error().message()))
         else:
-            ts = (reply.argumentAt(0).toPyObject()[0].toULongLong()[0],
+            ts_ = (reply.argumentAt(0).toPyObject()[0].toULongLong()[0],
                   reply.argumentAt(0).toPyObject()[1].toULongLong()[0])
             exports = []
             for export in reply.argumentAt(1).toPyObject():
                 ex = export.toPyObject()
                 lasttime = ex[10].toPyObject()
-                exp = Export(ExportID = ex[0].toInt()[0],
-                             ExportPath = str(ex[1].toString()),
-                             HasNFSv3 = ex[2].toBool(),
-                             HasMNT = ex[3].toBool(),
-                             HasNLM4 = ex[4].toBool(),
-                             HasRQUOTA = ex[5].toBool(),
-                             HasNFSv40 = ex[6].toBool(),
-                             HasNFSv41 = ex[7].toBool(),
-                             HasNFSv42 = ex[8].toBool(),
-                             Has9P = ex[9].toBool(),
-                             LastTime = (lasttime[0].toPyObject(),
-                                         lasttime[1].toPyObject()))
+                exp = Export(ExportID=ex[0].toInt()[0],
+                             ExportPath=str(ex[1].toString()),
+                             HasNFSv3=ex[2].toBool(),
+                             HasMNT=ex[3].toBool(),
+                             HasNLM4=ex[4].toBool(),
+                             HasRQUOTA=ex[5].toBool(),
+                             HasNFSv40=ex[6].toBool(),
+                             HasNFSv41=ex[7].toBool(),
+                             HasNFSv42=ex[8].toBool(),
+                             Has9P=ex[9].toBool(),
+                             LastTime=(lasttime[0].toPyObject(),
+                                       lasttime[1].toPyObject()))
                 exports.append(exp)
-            self.show_exports.emit(ts, exports)
+            self.show_exports.emit(ts_, exports)
 
 
 class ExportStats(QtDBus.QDBusAbstractInterface):
@@ -148,23 +148,23 @@ class ExportStats(QtDBus.QDBusAbstractInterface):
         self.stats_handler = stats_handler
 
     def GetNFSv3IO(self, exportid):
-        async = self.asyncCall("GetNFSv3IO", exportid)
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("GetNFSv3IO", exportid)
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.io_done)
 
     def GetNFSv40IO(self, exportid):
-        async = self.asyncCall("GetNFSv40IO", exportid)
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("GetNFSv40IO", exportid)
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.io_done)
 
     def GetNFSv41IO(self, exportid):
-        async = self.asyncCall("GetNFSv41IO", exportid)
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("GetNFSv41IO", exportid)
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.io_done)
 
     def GetNFSv41Layouts(self, exportid):
-        async = self.asyncCall("GetNFSv41Layouts", exportid)
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("GetNFSv41Layouts", exportid)
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.layout_done)
 
     def io_done(self, call):
@@ -172,4 +172,4 @@ class ExportStats(QtDBus.QDBusAbstractInterface):
 
     def layout_done(self, call):
         pass
-    
+
