@@ -879,6 +879,25 @@ void prune_defunct_exports(uint64_t generation)
 	release_root_op_context();
 }
 
+/**
+ * @brief Initialize all stats at startup time.
+ *
+ * Note: This function needs to be in all builds, not just USE_DBUS enabled
+ * ones.
+ *
+ */
+
+void nfs_init_stats_time(void)
+{
+	now(&nfs_stats_time);
+	fsal_stats_time = v3_full_stats_time
+			= v4_full_stats_time
+			= auth_stats_time
+			= clnt_allops_stats_time
+			= nfs_stats_time;
+}
+
+
 #ifdef USE_DBUS
 
 /* DBUS interfaces
@@ -1036,7 +1055,7 @@ static bool gsh_export_addexport(DBusMessageIter *args,
 		goto out;
 
 	config_struct = config_ParseFile(file_path, &err_type);
-	if (!config_error_is_harmless(&err_type)) {
+	if (!cur_exp_config_error_is_harmless(&err_type)) {
 		err_detail = err_type_str(&err_type);
 		LogCrit(COMPONENT_EXPORT,
 			"Error while parsing %s", file_path);
@@ -1080,7 +1099,8 @@ static bool gsh_export_addexport(DBusMessageIter *args,
 						   NULL,
 						   false,
 						   &err_type);
-			if (rc == 0 || config_error_is_harmless(&err_type))
+			if (rc == 0 ||
+			    cur_exp_config_error_is_harmless(&err_type))
 				exp_cnt++;
 			else if (!err_type.exists)
 				status = false;
@@ -1589,7 +1609,7 @@ static bool gsh_export_update_export(DBusMessageIter *args,
 		goto out;
 
 	config_struct = config_ParseFile(file_path, &err_type);
-	if (!config_error_is_harmless(&err_type)) {
+	if (!cur_exp_config_error_is_harmless(&err_type)) {
 		err_detail = err_type_str(&err_type);
 		LogCrit(COMPONENT_EXPORT,
 			"Error while parsing %s", file_path);
@@ -1633,7 +1653,8 @@ static bool gsh_export_update_export(DBusMessageIter *args,
 						   NULL,
 						   false,
 						   &err_type);
-			if (rc == 0 || config_error_is_harmless(&err_type))
+			if (rc == 0 ||
+			    cur_exp_config_error_is_harmless(&err_type))
 				exp_cnt++;
 			else if (!err_type.exists)
 				status = false;
@@ -1997,12 +2018,15 @@ static bool show_cache_inode_stats(DBusMessageIter *args,
 	bool success = true;
 	char *errormsg = "OK";
 	DBusMessageIter iter;
+	struct timespec timestamp;
 
+	now(&timestamp);
 	dbus_message_iter_init_append(reply, &iter);
 	gsh_dbus_status_reply(&iter, success, errormsg);
-	gsh_dbus_append_timestamp(&iter, &nfs_ServerBootTime);
+	gsh_dbus_append_timestamp(&iter, &timestamp);
 
 	mdcache_dbus_show(&iter);
+	mdcache_utilization(&iter);
 
 	return true;
 }
@@ -2058,9 +2082,7 @@ static bool stats_reset(DBusMessageIter *args,
 	reset_auth_stats();
 
 	/* update the stats counting time */
-	now(&nfs_stats_time);
-	fsal_stats_time = v3_full_stats_time = v4_full_stats_time =
-		auth_stats_time = nfs_stats_time;
+	nfs_init_stats_time();
 	return true;
 }
 
@@ -2542,6 +2564,7 @@ static struct gsh_dbus_method fsal_statistics = {
 		 END_ARG_LIST}
 };
 
+
 #ifdef _USE_9P
 /**
  * DBUS method to report 9p I/O statistics
@@ -2671,6 +2694,7 @@ static struct gsh_dbus_method cache_inode_show = {
 	.args = {STATUS_REPLY,
 		 TIMESTAMP_REPLY,
 		 TOTAL_OPS_REPLY,
+		 LRU_UTILIZATION_REPLY,
 		 END_ARG_LIST}
 };
 
@@ -2749,7 +2773,9 @@ static struct gsh_dbus_method *export_stats_methods[] = {
 	&status_stats,
 	&v3_full_statistics,
 	&v4_full_statistics,
+#ifdef _HAVE_GSSAPI
 	&auth_statistics,
+#endif /* _HAVE_GSSAPI */
 	&export_details,
 	NULL
 };
