@@ -67,6 +67,7 @@ struct fsal_obj_handle *nfs3_FhandleToCache(nfs_fh3 *fh3,
 	struct fsal_export *export;
 	struct fsal_obj_handle *obj = NULL;
 	struct gsh_buffdesc fh_desc;
+	char fhbuf[NFS3_FHSIZE];
 
 	/* Default behaviour */
 	*rc = NFS_REQ_OK;
@@ -84,9 +85,15 @@ struct fsal_obj_handle *nfs3_FhandleToCache(nfs_fh3 *fh3,
 
 	export = op_ctx->fsal_export;
 
-	/* Give the export a crack at it */
+	/*
+	 * FIXME: the wire handle can obviously be no larger than NFS4_FHSIZE,
+	 * but there is no such limit on a host handle. Here, we assume that as
+	 * the size limit. Eventually it might be nice to call into the FSAL to
+	 * ask how large a buffer it needs for a host handle.
+	 */
+	memcpy(fhbuf, &v3_handle->fsopaque, v3_handle->fs_len);
 	fh_desc.len = v3_handle->fs_len;
-	fh_desc.addr = &v3_handle->fsopaque;
+	fh_desc.addr = fhbuf;
 
 	/* adjust the wire handle opaque into a host-handle */
 	fsal_status =
@@ -401,157 +408,6 @@ int nfs3_Is_Fh_Invalid(nfs_fh3 *fh3)
 	return NFS3_OK;
 }				/* nfs3_Is_Fh_Invalid */
 
-/**
- * @brief Print an NFSv3 file handle
- *
- * @param[in] component Subsystem component ID
- * @param[in] fh        File handle to prin
- */
-void print_fhandle3(log_components_t component, nfs_fh3 *fh)
-{
-	if (isFullDebug(component)) {
-		char str[LEN_FH_STR];
-
-		sprint_fhandle3(str, fh);
-		LogFullDebug(component, "%s", str);
-	}
-}
-
-void sprint_fhandle3(char *str, nfs_fh3 *fh)
-{
-	char *tmp =
-	    str + sprintf(str, "File Handle V3: Len=%u ", fh->data.data_len);
-
-	sprint_mem(tmp, fh->data.data_val, fh->data.data_len);
-}
-
-/**
- * @brief Print an NFSv4 file handle
- *
- * @param[in] component Subsystem component ID
- * @param[in] fh        File handle to print
- */
-void print_fhandle4(log_components_t component, nfs_fh4 *fh)
-{
-	if (isFullDebug(component)) {
-		char str[LEN_FH_STR];
-
-		sprint_fhandle4(str, fh);
-		LogFullDebug(component, "%s", str);
-	}
-}
-
-void sprint_fhandle4(char *str, nfs_fh4 *fh)
-{
-	char *tmp =
-	    str + sprintf(str, "File Handle V4: Len=%u ", fh->nfs_fh4_len);
-
-	sprint_mem(tmp, fh->nfs_fh4_val, fh->nfs_fh4_len);
-}
-
-/**
- * @brief Print an NLM netobj
- *
- * @param[in] component Subsystem component ID
- * @param[in] fh        File handle to print
- */
-void print_fhandle_nlm(log_components_t component, netobj *fh)
-{
-	if (isFullDebug(component)) {
-		char str[LEN_FH_STR];
-
-		sprint_fhandle_nlm(str, fh);
-		LogFullDebug(component, "%s", str);
-	}
-}				/* print_fhandle_nlm */
-
-void sprint_fhandle_nlm(char *str, netobj *fh)
-{
-	char *tmp = str + sprintf(str, "File Handle V3: Len=%u ", fh->n_len);
-
-	sprint_mem(tmp, fh->n_bytes, fh->n_len);
-}				/* sprint_fhandle_nlm */
-
-/**
- * @brief Print the content of a buffer.
- *
- * @param[in] component Logging subsystem ID
- * @param[in] buff      Buffer to print.
- * @param[in] len       Length of the buffer.
- */
-void print_buff(log_components_t component, char *buff, int len)
-{
-	if (isFullDebug(component)) {
-		char str[1024];
-
-		sprint_buff(str, buff, len);
-		LogFullDebug(component, "%s", str);
-	}
-}				/* print_buff */
-
-void sprint_buff(char *str, char *buff, int len)
-{
-	char *tmp = str + sprintf(str, "  Len=%u Buff=%p Val: ", len, buff);
-
-	sprint_mem(tmp, buff, len);
-}				/* sprint_buff */
-
-void sprint_mem(char *str, char *buff, int len)
-{
-	int i;
-
-	if (buff == NULL)
-		sprintf(str, "<null>");
-	else
-		for (i = 0; i < len; i++)
-			sprintf(str + i * 2, "%02x", (unsigned char)buff[i]);
-}
-
-/**
- * @brief Convert a file handle to a string representation
- *
- * @param rq_vers  [IN]    version of the NFS protocol to be used
- * @param fh3      [IN]    NFSv3 file handle or NULL
- * @param fh4      [IN]    NFSv4 file handle or NULL
- * @param str      [OUT]   string version of handle
- *
- */
-void nfs_FhandleToStr(u_long rq_vers, nfs_fh3 *fh3, nfs_fh4 *fh4, char *str)
-{
-
-	switch (rq_vers) {
-	case NFS_V4:
-		sprint_fhandle4(str, fh4);
-		break;
-
-	case NFS_V3:
-		sprint_fhandle3(str, fh3);
-		break;
-	}
-}				/* nfs_FhandleToStr */
-
-/**
- *
- * print_compound_fh
- *
- * This routine prints all the file handle within a compoud request's data
- * structure.
- *
- * @param data [IN] compound's data to manage.
- */
-void LogCompoundFH(compound_data_t *data)
-{
-	if (isFullDebug(COMPONENT_FILEHANDLE)) {
-		char str[LEN_FH_STR];
-
-		sprint_fhandle4(str, &data->currentFH);
-		LogFullDebug(COMPONENT_FILEHANDLE, "Current FH  %s", str);
-
-		sprint_fhandle4(str, &data->savedFH);
-		LogFullDebug(COMPONENT_FILEHANDLE, "Saved FH    %s", str);
-	}
-}
-
 /**
  * @brief Do basic checks on the CurrentFH
  *
@@ -667,7 +523,6 @@ nfsstat4 nfs4_sanity_check_saved_FH(compound_data_t *data, int required_type,
 						data->current_filetype));
 			if (-required_type == DIRECTORY) {
 				return NFS4ERR_ISDIR;
-				return NFS4ERR_INVAL;
 			}
 		}
 	} else if (required_type != NO_FILE_TYPE &&
