@@ -102,33 +102,27 @@ int compare_ip_name(struct gsh_buffdesc *buff1, struct gsh_buffdesc *buff2)
 /**
  * @brief Display the ip_name stored in the buffer
  *
- * @param[in]  pbuff Buffer to display
- * @param[out] str   Output string
- *
- * @return number of character written.
+ * @param[in]  dspbuf display buffer to display into
+ * @param[in]  buff   Buffer to display
  */
-int display_ip_name_key(struct gsh_buffdesc *pbuff, char *str)
+int display_ip_name_key(struct display_buffer *dspbuf,
+			struct gsh_buffdesc *buff)
 {
-	sockaddr_t *addr = (sockaddr_t *) (pbuff->addr);
-
-	sprint_sockip(addr, str, HASHTABLE_DISPLAY_STRLEN);
-	return strlen(str);
+	return display_sockip(dspbuf, buff->addr);
 }
 
 /**
  * @brief Display the ip_name stored in the buffer
  *
- * @param[in]  pbuff Buffer to display
- * @param[out] str   Output string
- *
- * @return number of character written
+ * @param[in]  dspbuf display buffer to display into
+ * @param[in]  buff   Buffer to display
  */
-int display_ip_name_val(struct gsh_buffdesc *pbuff, char *str)
+int display_ip_name_val(struct display_buffer *dspbuf,
+			struct gsh_buffdesc *buff)
 {
-	nfs_ip_name_t *nfs_ip_name = (pbuff->addr);
+	nfs_ip_name_t *nfs_ip_name = (buff->addr);
 
-	return snprintf(str, HASHTABLE_DISPLAY_STRLEN, "%s",
-			nfs_ip_name->hostname);
+	return display_cat(dspbuf, nfs_ip_name->hostname);
 }
 
 /**
@@ -143,42 +137,33 @@ int display_ip_name_val(struct gsh_buffdesc *pbuff, char *str)
  * @return IP_NAME_SUCCESS if successful
  * @return IP_NAME_INSERT_MALLOC_ERROR if an error occurred during the insertion
  *                                     process
- * @return IP_NAME_NETDB_ERROR if an error occurred during the netdb query (via
- *                             gethostbyaddr).
  *
  */
 
-int nfs_ip_name_add(sockaddr_t *ipaddr, char *hostname, size_t size)
+int nfs_ip_name_add(sockaddr_t *ipaddr, char *hostname, size_t maxsize)
 {
 	struct gsh_buffdesc buffkey;
 	struct gsh_buffdesc buffdata;
 	nfs_ip_name_t *nfs_ip_name = NULL;
-	sockaddr_t *pipaddr = NULL;
 	struct timeval tv0, tv1, dur;
-	int rc;
-	char ipstring[SOCK_NAME_MAX + 1];
+	int rc, len, size;
+	char ipstring[SOCK_NAME_MAX];
+	char *hn = hostname;
 	hash_error_t hash_rc;
 
-	nfs_ip_name = gsh_malloc(sizeof(nfs_ip_name_t));
-
-	pipaddr = gsh_malloc(sizeof(sockaddr_t));
-
-	/* I have to keep an integer as key, I wil use the pointer buffkey->addr
-	 * for this, this also means that buffkey->len will be 0
-	 */
-	memcpy(pipaddr, ipaddr, sizeof(sockaddr_t));
-
-	buffkey.addr = pipaddr;
-	buffkey.len = sizeof(sockaddr_t);
-
 	gettimeofday(&tv0, NULL);
-	rc = getnameinfo((struct sockaddr *)pipaddr, sizeof(sockaddr_t),
-			 nfs_ip_name->hostname, sizeof(nfs_ip_name->hostname),
+
+	/* Speculatively get the hostname into our caller's buffer... */
+	rc = getnameinfo((struct sockaddr *)ipaddr, sizeof(sockaddr_t),
+			 hostname, maxsize,
 			 NULL, 0, 0);
 	gettimeofday(&tv1, NULL);
 	timersub(&tv1, &tv0, &dur);
 
-	sprint_sockip(pipaddr, ipstring, sizeof(ipstring));
+	if (!sprint_sockip(ipaddr, ipstring, sizeof(ipstring))) {
+		/* Error in converting socket address into string. */
+		return IP_NAME_INSERT_MALLOC_ERROR;
+	}
 
 	/* display warning if DNS resolution took more that 1.0s */
 	if (dur.tv_sec >= 1) {
@@ -190,51 +175,70 @@ int nfs_ip_name_add(sockaddr_t *ipaddr, char *hostname, size_t size)
 
 	/* Ask for the name to be cached */
 	if (rc != 0) {
-		strmaxcpy(nfs_ip_name->hostname, ipstring,
-			sizeof(nfs_ip_name->hostname));
+		hn = ipstring;
 		LogEvent(COMPONENT_DISPATCH,
-			 "Cannot resolve address %s, error %s, using %s as hostname",
-			 ipstring, gai_strerror(rc), nfs_ip_name->hostname);
+			 "Cannot resolve address %s, error %s, using address as hostname",
+			 ipstring, gai_strerror(rc));
+
+		if (maxsize < SOCK_NAME_MAX) {
+			LogMajor(COMPONENT_DISPATCH,
+				 "Could not return ip address because caller's buffer was too small");
+			return IP_NAME_INSERT_MALLOC_ERROR;
+		}
+
+		/* And copy the ipstring out to the caller's buffer. */
+		strcpy(hostname, ipstring);
 	}
 
-	LogDebug(COMPONENT_DISPATCH, "Inserting %s->%s to addr cache", ipstring,
-		 nfs_ip_name->hostname);
-
-	/* I build the data with the request pointer
-	 * that should be in state 'IN USE'
+	/* At this point, no matter what, the caller's buffer has been filled
+	 * with the hostname we would cache.
 	 */
+
+	/* I have to keep an integer as key, I wil use the pointer buffkey->addr
+	 * for this, this also means that buffkey->len will be 0
+	 */
+	buffkey.len = sizeof(sockaddr_t);
+	buffkey.addr = gsh_memdup(ipaddr, buffkey.len);
+
+	/* Now setup the cached hostname */
+	len = strlen(hn);
+	size = sizeof(nfs_ip_name_t) + len + 1;
+
+	nfs_ip_name = gsh_malloc(size);
+
 	nfs_ip_name->timestamp = time(NULL);
+	memcpy(nfs_ip_name->hostname, hn, len + 1);
+
+	LogDebug(COMPONENT_DISPATCH, "Inserting %s->%s to addr cache",
+		 ipstring, hn);
 
 	buffdata.addr = nfs_ip_name;
-	buffdata.len = sizeof(nfs_ip_name_t);
+	buffdata.len = size;
 
 	/* Multiple threads may try to add the same IP/name to cache. Need to
 	 * return success if we get HASHTABLE_ERROR_KEY_ALREADY_EXISTS
 	 * as return status
 	 */
 	hash_rc = HashTable_Set(ht_ip_name, &buffkey, &buffdata);
-	switch (hash_rc) {
-	case HASHTABLE_SUCCESS:
-		/* Copy the value for the caller */
-		strmaxcpy(hostname, nfs_ip_name->hostname, size);
-		return IP_NAME_SUCCESS;
-	case HASHTABLE_ERROR_KEY_ALREADY_EXISTS:
-		/* Copy the value for the caller */
-		strmaxcpy(hostname, nfs_ip_name->hostname, size);
+
+	/* No matter if we were able to cache or not, we either have a hostname
+	 * or it didn't work, so we will return the hostname from above which is
+	 * already in the caller's buffer.
+	 */
+	if (hash_rc != HASHTABLE_SUCCESS) {
+		if (hash_rc != HASHTABLE_ERROR_KEY_ALREADY_EXISTS) {
+			/* This should not happen */
+			LogEvent(COMPONENT_DISPATCH,
+				 "Error %s while adding host %s to cache",
+				 hash_table_err_to_str(hash_rc), hn);
+		}
+
 		/* Release not required allocations */
 		gsh_free(nfs_ip_name);
-		gsh_free(pipaddr);
-		return IP_NAME_SUCCESS;
-	default:
-		/* This should not happen */
-		LogEvent(COMPONENT_DISPATCH,
-			 "Error while adding host %s to cache",
-			 nfs_ip_name->hostname);
-		/* Release not required allocations */
-		gsh_free(nfs_ip_name);
-		gsh_free(pipaddr);
-		return IP_NAME_INSERT_MALLOC_ERROR;
+		gsh_free(buffkey.addr);
 	}
+
+	return IP_NAME_SUCCESS;
 }				/* nfs_ip_name_add */
 
 /**
@@ -254,9 +258,12 @@ int nfs_ip_name_get(sockaddr_t *ipaddr, char *hostname, size_t size)
 	struct gsh_buffdesc buffkey;
 	struct gsh_buffdesc buffval;
 	nfs_ip_name_t *nfs_ip_name;
-	char ipstring[SOCK_NAME_MAX + 1];
+	char ipstring[SOCK_NAME_MAX];
 
-	sprint_sockip(ipaddr, ipstring, sizeof(ipstring));
+	if (!sprint_sockip(ipaddr, ipstring, sizeof(ipstring))) {
+		/* Error in converting socket address into string. */
+		return IP_NAME_NOT_FOUND;
+	}
 
 	buffkey.addr = ipaddr;
 	buffkey.len = sizeof(sockaddr_t);
@@ -265,7 +272,12 @@ int nfs_ip_name_get(sockaddr_t *ipaddr, char *hostname, size_t size)
 			  &buffkey,
 			  &buffval) == HASHTABLE_SUCCESS) {
 		nfs_ip_name = buffval.addr;
-		strmaxcpy(hostname, nfs_ip_name->hostname, size);
+		if (strlcpy(hostname, nfs_ip_name->hostname, size) >= size) {
+			LogWarn(COMPONENT_DISPATCH,
+				"Could not return host %s to caller, too big",
+				nfs_ip_name->hostname);
+			return IP_NAME_INSERT_MALLOC_ERROR;
+		}
 
 		LogFullDebug(COMPONENT_DISPATCH, "Cache get hit for %s->%s",
 			     ipstring, nfs_ip_name->hostname);
@@ -293,9 +305,12 @@ int nfs_ip_name_remove(sockaddr_t *ipaddr)
 {
 	struct gsh_buffdesc buffkey, old_value;
 	nfs_ip_name_t *nfs_ip_name = NULL;
-	char ipstring[SOCK_NAME_MAX + 1];
+	char ipstring[SOCK_NAME_MAX];
 
-	sprint_sockip(ipaddr, ipstring, sizeof(ipstring));
+	if (!sprint_sockip(ipaddr, ipstring, sizeof(ipstring))) {
+		/* Error in converting socket address into string. */
+		return IP_NAME_NOT_FOUND;
+	}
 
 	buffkey.addr = ipaddr;
 	buffkey.len = sizeof(sockaddr_t);
@@ -353,8 +368,8 @@ static struct ip_name_cache ip_name_cache = {
 	.hash_param.hash_func_key = ip_name_value_hash_func,
 	.hash_param.hash_func_rbt = ip_name_rbt_hash_func,
 	.hash_param.compare_key = compare_ip_name,
-	.hash_param.key_to_str = display_ip_name_key,
-	.hash_param.val_to_str = display_ip_name_val,
+	.hash_param.display_key = display_ip_name_key,
+	.hash_param.display_val = display_ip_name_val,
 	.hash_param.flags = HT_FLAG_NONE,
 };
 
