@@ -37,11 +37,11 @@ char usage[] =
 	"       ml_posix_client -x script [-q] [-d] [-c path]\n"
 	"       ml_posix_client [-q] [-d] [-c path]\n" "\n"
 	"  ml_posix_client may be run in three modes\n"
-	"  - In the first mode, the client will be driven by a master.\n"
+	"  - In the first mode, the client will be driven by a console.\n"
 	"  - In the second mode, the client is driven by a script.\n"
 	"  - In the third mode, the client interractive.\n" "\n"
-	"  -s server - specify the master's hostname or IP address\n"
-	"  -p port   - specify the master's port number\n"
+	"  -s server - specify the console's hostname or IP address\n"
+	"  -p port   - specify the console's port number\n"
 	"  -n name   - specify the client's name\n"
 	"  -x script - specify the name of a script to execute\n"
 	"  -q        - specify quiet mode\n"
